/* -*- Mode: C++ -*-
 * $Id: FuzzyFooDecision.h,v 1.2 2001/07/06 13:53:08 sigge Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: FuzzyFooDecision.h
 * Unitname: Framework/example/FuzzyFoo
 * $Revision: 1.2 $
 * Created by: Mikael Brnnstrm, Andreas Bckert & Andreas Sigfridsson
 * Last modified by $Author: sigge $ $Date: 2001/07/06 13:53:08 $
 *
 *
 * HISTORY
 *
 * $Log: FuzzyFooDecision.h,v $
 * Revision 1.2  2001/07/06 13:53:08  sigge
 * Removed IntentionDecision from various places.
 *
 * Revision 1.1  2001/06/19 14:30:24  mikbr212
 * Added team FuzzyFoo
 *
 *
 */

/**
 * @file FuzzyFooDecision.h
 * Include file for the class FuzzyFooDecision.
 *
 * @par Unit:
 *      Agent/FuzzyFoo
 * $Revision: 1.2 $
 * $Author: sigge $
 * $Date: 2001/07/06 13:53:08 $
 */


#ifndef FCFOODECISION_H
#define FCFOODECISION_H


#define RS_USE_DISPLAY
//#define RS_USE_COACH_DEBUGGING


#include "basic_system_compiler_directives.h"
#include "Decision.h"
#include "ViewManager.h"

#include "Skill.h"

#ifdef RS_USE_DISPLAY
#  include "Display.h"
#endif

#ifdef RS_USE_COACH_DEBUGGING
//  #  include "CoachView.h"
#endif


RS_BEGIN_NAMESPACE

class CommunicationView;
class GameView;
class LookView;
class MarkerView;
class ParameterView;
class PassTreeView;
class WorldStateView;

class Intention;

/**
 * @class FuzzyFooDecision FuzzyFooDecision.h
 * Example class for what a decision maker might look like.
 *
 * <!-- @todo-->
 *
 * @par Unit:
 *      Library
 * $Revision: 1.2 $
 * $Author: sigge $
 * $Date: 2001/07/06 13:53:08 $
 */
class FuzzyFooDecision : public Decision {
public:
  FuzzyFooDecision(ActuatorInterface* const actuator_interface = NULL);
  virtual ~FuzzyFooDecision();

  virtual ServerCommand* initAgent();
  virtual bool start();
  //virtual bool init(ActuatorInterface* const actuator_interface);
  virtual bool decide();

  // Event handlers
  virtual void onActuatorSensorData();
  //virtual void onAuralSensorData();
  virtual void onInit();
  //virtual void onPhysicalSensorData();
  virtual void onVisualSensorData();

  //virtual void onBeforeSensorData();
  virtual void onAfterSensorData();
  //virtual void onBeforeTick();
  //virtual void onAfterTick();

  virtual void onNewCycle(const bool forced);
  virtual bool onEstimatedNewCycle(UInt16 estimated_time, UInt16 ticks);
  virtual void onCommandWarning();

protected:
  Intention* chooseIntention();

  ViewManager* const viewManager;
  ParameterView* parameterView;
  GameView* gameView;
  MarkerView* markerView;
  CommunicationView* communicationView;
  WorldStateView* worldStateView;
  LookView* lookView;
  PassTreeView* passTreeView;

  bool serverAlive;
  bool busy;
  std::vector<Intention*> intentions;

#ifdef RS_USE_DISPLAY
  Display* theDisplay;
#endif

#ifdef RS_USE_COACH_DEBUGGING
//    CoachView* coachView;
#endif
};


RS_END_NAMESPACE


#endif // !FCFOODECISION_H
