/* -*- Mode: C++ -*-
 * $Id: Display.h,v 1.2 2001/04/11 16:07:48 andbo928 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: Display.h
 * Unitname: Framework/example/FCFoo
 * $Revision: 1.2 $
 * Created by: Johan Thapper 1999-xx-xx
 * Last modified by $Author: andbo928 $ $Date: 2001/04/11 16:07:48 $
 *
 *
 * HISTORY
 *
 * $Log: Display.h,v $
 * Revision 1.2  2001/04/11 16:07:48  andbo928
 * Fixed display to show a lot of thingys...
 *
 * Revision 1.1.1.1  2001/04/08 11:33:58  sigge
 * Imported sources
 *
 * Revision 1.3  2000/10/24 15:15:45  frehe
 * Made minor updates and tests.
 *
 * Revision 1.2  2000/10/11 18:40:29  frehe
 * Upgraded FCFoo.
 *
 * Revision 1.1  2000/09/05 21:12:54  frehe
 * Made FCFoo compile with RoboSoc 2.5.0
 *
 */

/**
 * @file Display.h
 * Include file for the class Display.
 *
 * @par Unit:
 *      Framework/example/FCFoo
 * $Revision: 1.2 $
 * $Author: andbo928 $
 * $Date: 2001/04/11 16:07:48 $
 */

#include "TVector.h"


#ifndef DISPLAY_H
#define DISPLAY_H


//#define RS_USE_COACH_DEBUGGING


#include <string>
#include <fstream>

#include "ViewManager.h"
#include "ParameterView.h"
#include "GameView.h"
#include "MarkerView.h"
#include "AgentView.h"
#include "BallView.h"
#include "PlayerView.h"
#ifdef RS_USE_COACH_DEBUGGING
#  include "CoachView.h"
#endif


#define CLEAR_SCREEN cout << char(27) << "[2J"
#define HOME_CURSOR cout << char(27) << "[H"
#define MOVE_CURSOR(x,y) cout << char(27) << '[' << x << ',' << y << 'H'


RS_BEGIN_NAMESPACE


/**
 * @class Display Display.h
 * Example class for what a debug information display could look like.
 *
 * <!-- @todo-->
 *
 * @par Unit:
 *      Library
 * $Revision: 1.2 $
 * $Author: andbo928 $
 * $Date: 2001/04/11 16:07:48 $
 */
class Display {
public:
  Display(UInt x = 40, UInt y = 10);
  ~Display();
  
  void setLastSkill(const std::string&);
  void draw();
  void addArrow(const TVector& start, 
		const TVector& end, 
		double duration = 0.1,
		string color = "black");
  void addMessage(string msg);

protected:
  ViewManager* viewManager;
  ParameterView* parameterView;
  GameView* gameView;
  MarkerView* markerView;
  AgentView* agentView;
  BallView* ballView;
  PlayerView* playerView;
#ifdef RS_USE_COACH_DEBUGGING
  CoachView* coachView;
#endif
  
  UInt col;
  UInt row;

  vector<TVector> arrowStart;
  vector<TVector> arrowEnd;
  vector<double> arrowDuration;
  vector<string> arrowColor;

  vector<string> messages;

  std::string lastSkill;
  std::fstream file;
};


RS_END_NAMESPACE


#endif // !DISPLAY_H

