#include "PassTree.h"

RS_BEGIN_NAMESPACE

/*
PlayerStruct::PlayerStruct() 
  : pos(Point(0,0)),
    value(0.0),
    parent(-1),
    no(0)
{
}
*/

PassTree::PassTree()
  : parameterView(NULL),
    agentView(NULL),
    playerView(NULL),
    ballView(NULL),
    passThreshold(0.5),
    maxBorrowAngle(5.0),
    bestAngle(30.0),
    minDist(2),
    bestDist(5),
    maxDist(20) //,
    //    predTree(NULL)
{
  getView(parameterView,PARAMETER_VIEW_ID);
  getView(agentView,AGENT_VIEW_ID,parameterView);
  getView(playerView,PLAYER_VIEW_ID);
  getView(ballView, BALL_VIEW_ID);
}

PassTree::~PassTree() {
  //  viewManager->releaseView(BALL_VIEW_ID);
  //  viewManager->releaseView(PLAYER_VIEW_ID);
  //  viewManager->releaseView(AGENT_VIEW_ID);
  //  viewManager->releaseView(PARAMETER_VIEW_ID);
}

void PassTree::buildTree() {
  std::vector<Point> op,tm; // I use Point since I will do a lot of + and -
  std::vector<UInt16> tmNo;
  float passVal[11][11];  // [from][to]
  
  std::vector<PlayerObject> opObjs,tmObjs; 
  std::vector<PlayerObject>::iterator it;
  
  opObjs = playerView->getOpponents(NOW);
  tmObjs = playerView->getTeammates(NOW);
  
  for(it = opObjs.begin(); it != opObjs.end(); it++)
    if(it->getPosition().isKnown()) 
      op.push_back(it->getPosition().getPoint());
  
  tm.push_back(Point(0,0)); // add myself
  for(it = tmObjs.begin(); it != tmObjs.end(); it++)
    if(it->getPosition().isKnown()) {
      tm.push_back(it->getPosition().getPoint());
      
      if(it->getUniformNumber().isKnown())
	tmNo.push_back(it->getUniformNumber().getValue());
    }
  
  // calculate distance-value (it is symmetric)
  /* debug */
  if(tm.size() > 11)
    cout << "*** ERROR in PassTree::buildTree, tm.size > 11 ***\n";
  /* end debug*/
  unsigned int i,j;
  for(i = 0; i < tm.size(); i++) {
    passVal[i][i] = 0; // don't pass to myself
    for(j = 0; j < i; j++) 
      passVal[i][j] = passVal[j][i] = distFn(tm[i],tm[j]);
  }
  
  // calculate angle-to-opponents-value for non-zero distance-values
  for(i = 0; i < tm.size(); i++) 
    for(j = 0; j < tm.size(); j++) 
      if(passVal[i][j] > 0) 
	passVal[i][j] *= angleFn(tm[i],tm[j],op);
  
  // now use Dijkstra to get best way from the root(s)
  std::vector<unsigned int> visited;
  std::vector<PlayerStruct> tree;
  unsigned int current;
  predTree = &tree; // make available for predicate

  PlayerStruct player;
  player.parent = -1;
  player.value = 0.0;
  for(i = 0; i < tm.size(); i++) {
    player.pos = tm[i];
    player.no = tmNo[i];
    tree.push_back(player);
  }

  visited = getRoot(tree); 
  make_heap(visited.begin(),visited.end(),&heapPredicate);
  while(!visited.empty()) {
    current = popHeap(visited);
    for(i = 0; i < tm.size(); i++) {
      if(tree[current].value * passVal[current][i] > tree[i].value) {
	tree[i].value = tree[current].value * passVal[current][i];
	pushHeap(visited,i);
      }
    }
  }
  predTree = NULL;
  // now tree is built -> find my parent (and children)
}

bool PassTree::heapPredicate(unsigned int x, unsigned int y) {
  /* debug */
  if(x >= predTree->size() || y >= predTree->size())
    cout << "ERROR PassTree::heapPredicate, x or y >= tree.size()!!\n";
  /* end debug */

  return ((*predTree)[x].value < (*predTree)[y].value);
}

void PassTree::pushHeap(std::vector<unsigned int>& h, unsigned int x) {
  h.push_back(x);
  push_heap(h.begin(),h.end(),heapPredicate);
}

unsigned int PassTree::popHeap(std::vector<unsigned int>& h) {
  pop_heap(h.begin(),h.end(),heapPredicate);
  unsigned int retVal = h.back();
  h.pop_back();
  // remove duplicates
  while(!h.empty() && h.front() == retVal) {
    pop_heap(h.begin(),h.end(),heapPredicate);
    h.pop_back();
  }
  return retVal;
}

// This implementation only choose the player nearest the ball
std::vector<unsigned int> PassTree::getRoot(std::vector<PlayerStruct>& tree) {
  unsigned int closest,i;
  float minDist = 200.0;
  float dist;
  std::vector<unsigned int> retVal;
  if(ballView->getRelativeBallVector(NOW).isUnknown())
    return retVal; // no root

  for(i = 0; i < tree.size(); i++) {
    dist = sqrt(pow(tree[i].pos.getX(),2) +
		pow(tree[i].pos.getY(),2));
    if(dist < minDist) {
      minDist = dist;
      closest = i;
    }
  }

  retVal.push_back(closest); 
  tree[closest].value = 1.0;
  return retVal;
}  

float PassTree::distFn(const Point& from, const Point& to) {
  Point p = from - to;
  float dist = sqrt(pow(p.getX(),2) + pow(p.getY(),2));
  return distFn(dist);
}

float PassTree::distFn(float dist) {
  // dist should be positive and in meters

  // linear interpolation between these points:
  // (0,0),(minDist,1),(bestDist,1),(maxDist,0),(inf,0)

  if(dist < minDist) 
    return dist/minDist;

  if(dist < bestDist)
    return 1.0;

  if(dist < maxDist)
    return 1.0 - (dist - bestDist)/(maxDist - bestDist);

  return 0.0;
}

float PassTree::angleFn(const Point& from, const Point& to,
			std::vector<Point>& op) {
  float rAngle = 90;
  float lAngle = 90;
  float angle = 0;
  Vector passVec = Vector(from - to);
  
  // find angle to closest opponents
  std::vector<Point>::iterator it;
  Vector opVec(0,0);
  for(it = op.begin(); it != op.end(); it++) {
    opVec = Vector(*it - from);
    
    // skip opponents to far away
    if(opVec.getLength() <= 1.1*passVec.getLength()) {
      angle = (passVec.getAngle() - opVec.getAngle()).getDeg();
      // is opponent to the right or to the left?
      if(angle > 0 && angle < rAngle)
	rAngle = angle;
      else if(angle <= 0 && -angle < lAngle)
	lAngle = -angle;
    }
  } // for
  
  angle = min(rAngle,lAngle);
  if(angle + maxBorrowAngle < (rAngle+lAngle)/2)
    angle+=maxBorrowAngle;
  else
    angle = (rAngle+lAngle)/2;
  
  return angleFn(angle);
}

float PassTree::angleFn(float angle) {
  // angle should be positive and in degrees

  if(angle > bestAngle)
    return 1.0;
  
  return sin(180*angle/bestAngle - 90);
}
/*
float PassTree::passValue(PlayerObject pl, Vector to) {
  if(pl.getRelativeVectorTo().isUnknown())
    return 0.0;
  return passValue(pl.getRelativeVectorTo().getVector(),to);
}

float PassTree::passValue(Vector from, Vector to) {
  Vector passVec = from - to;
  
  float rAngle = 90;
  float lAngle = 90;
  float angle = 0;

  std::vector<PlayerObject> op;
  op = playerView->getOpponents(NOW);
  std::vector<PlayerObject>.iterator it;

  // find angle to closest opponents
  Vector opVec = Vector(0,0);
  for(it = op.begin(); it != op.end(); it++) {
    if(it->getRelativeVectorTo().isKnown()) {
      opVec = it.getRelativeVectorTo().getVector() - from;
      
      // skip opponents to far away
      if(opVec.getLength() <= 1.1*passVec.getLength()) {
	
	angle = passVec.getAngle() - opVec.getAngle();
	// is opponent to the right or to the left?
	if(angle > 0 && angle < rAngle)
	  rAngle = angle;
	else if(angle <= 0 && -angle < lAngle)
	  lAngle = -angle;
      }
    }
  } // for

  angle = min(rAngle,lAngle);
  if(angle + maxBorrowAngle < (rAngle+lAngle)/2)
    angle+=maxBorrowAngle;
  else
    angle = (rAngle+lAngle)/2;
  
  return distFn(passVec.getLength()) * angleFn(angle);
}

*/

/*

PlayerStruct PassTree::getParent(Vector ref) {
  std::vector<PlayerObject> tm;
  tm = playerView->getTeammates(NOW);
  std::vector<PlayerObject>.iterator it;

  PlayerStruct theParent;

  float passValue;
  for(it = tm.begin(); it != tm.end(); it++) {
    passValue = passValue(*it,ref); 
    if(passValue >= passThreshold)
      // remember the best parent
      if(passValue > theParent.value) {
	if(it->getRelativeVectorTo().isKnown()) // should be known
	  theParent.pos = it->getRelativeVectorTo().getVector();
	if(it->getUniformNumber().isKnown())
	  theParent.no = it->getUniformNumber().getValue();
	theParent.value = passValue;
      }
    } // for
    
  return theParent; // if no parent found, x.pos = (0,0)
}

std::vector<PlayerStruct> PassTree::getChildren() {
  std::vector<PlayerObject> tm;
  tm = playerView->getTeammates(NOW);
  std::vector<PlayerObject>.iterator it;
  
  PlayerStruct theChild, parent;
  std::vector<PlayerStruct> myChildren;
  float passValue;
  for(it = tm.begin(); it != tm.end(); it++) {
    if(it->getRelativeVectorTo().isKnown()) {
      parent = getParent(it->getRelativeVectorTo().getVector());
      passValue = passValue(Vector(0,0),
			       it->getRelativeVectorTo().getVector());

      if(passValue > passThreshold && passValue > parent.value) {

	// add player to myChildren
	theChild.pos = it->getRelativeVectorTo().getVector();
	if(it->getUniformNumber().isKnown())
	  theChild.no = it->getUniformNumber().getValue();
	theChild.value = passValue;
	myChildren.push_back(theChild);
      }
    }
  } // for
  return myChildren;
}

*/

RS_END_NAMESPACE
