/* -*- Mode: C++ -*-
 * $Id: WeHaveBall.cc,v 1.2 2001/07/28 07:17:57 mikbr212 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: WeHaveBall.cc
 * Unitname: Framework/predicate
 * $Revision: 1.2 $
 * Created by: Fredrik Heintz 2000-09-20
 * Last modified by $Author: mikbr212 $ $Date: 2001/07/28 07:17:57 $
 *
 *
 * HISTORY
 *
 * $Log: WeHaveBall.cc,v $
 * Revision 1.2  2001/07/28 07:17:57  mikbr212
 * Removed warnings!!
 *
 * Revision 1.1  2001/04/17 14:55:27  andbo928
 * Created... might be buggy...
 *
 * Revision 1.2  2001/04/17 12:47:59  andbo928
 * It now uses worldstateview.
 *
 * Revision 1.1.1.1  2001/04/08 11:33:58  sigge
 * Imported sources
 *
 * Revision 1.2  2000/11/08 17:54:20  olalu526
 * This fixes a LOT of problems.
 *
 * Revision 1.1  2000/09/20 14:42:48  frehe
 * Added the predicate WeHaveBall that will return true iff the agent is the closest player to the ball.
 *
 */

/**
 * @file WeHaveBall.cc
 * Implementation file for the predicate class WeHaveBall.
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.2 $
 * $Author: mikbr212 $
 * $Date: 2001/07/28 07:17:57 $
 */


#include "WeHaveBall.h"
#include "view_constants.h"
#include "ViewManager.h"
#include "TVector.h"

RS_BEGIN_NAMESPACE

WeHaveBall::WeHaveBall()
  : Predicate(),
    worldView(NULL)
{
  getView(worldView, WORLD_STATE_VIEW_ID);
}

WeHaveBall::WeHaveBall(const WeHaveBall& obj)
  : Predicate(obj),
  worldView(NULL)
{
  getView(worldView, WORLD_STATE_VIEW_ID);
}

WeHaveBall::~WeHaveBall()
{
  viewManager->releaseView(worldView->getId());
}

WeHaveBall& WeHaveBall::operator= (const WeHaveBall&)
{

  // Don't copy the worldView pointer.
  // Only the predicate parameters if any.
  return *this;
}

double WeHaveBall::getDistToPlayers(const vector<PlayerState>& players,
				    TVector point)
{
  double closeDist = 10000.0;
  vector<PlayerState>::const_iterator it;
  for(it = players.begin(); it != players.end(); it++)
    {
      TVector delta = it->getPosition() - point;
      double dist = delta.getLength();
      if(dist < closeDist)
	{
	  closeDist = dist;
	}
    }
  return closeDist;
}

FuzzyBool WeHaveBall::evaluate(const WorldState& ws)
{
  // Predicate is true iff
  // We're closer to the ball then any opponent
  // We're closer than 10m from the ball.


  TVector ballPos = ws.getBall().getPosition();

  double ourDist = min((ballPos - ws.getAgent().getPosition()).getLength(),
		       getDistToPlayers(ws.getTeammates(),
					ballPos));

  // If our guys are to far away there is no need to process opponents.
  if(ourDist > 10)
    return FALSE;

  double theirDist = getDistToPlayers(ws.getOpponents(),
				      ballPos);

  // Check if the predicate is true.

  if(ourDist > theirDist)
    return FALSE;

  return TRUE;
}

FuzzyBool WeHaveBall::evaluate()
{
  return evaluate(worldView->getWorldState());
}

RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "WeHaveBall.icc"
#endif
