/* -*- Mode: C++ -*-
 * $Id: AdvanceBallIntention.cc,v 1.19 2001/08/01 14:17:00 mikbr212 Exp $
 *
 * IDENTIFICATION
 *
 * $Revision: 1.19 $
 * Created by: Mikael Brnnstrm, Andreas Bckert, Andreas Sigfridsson
 * Last modified by $Author: mikbr212 $ $Date: 2001/08/01 14:17:00 $
 *
 *
 * HISTORY
 *
 * $Log: AdvanceBallIntention.cc,v $
 * Revision 1.19  2001/08/01 14:17:00  mikbr212
 * Removed debug spam
 *
 * Revision 1.18  2001/07/31 16:31:11  sigge
 * Hardcoded passes after kick off and added dribble stuff.
 *
 * Revision 1.17  2001/07/31 11:10:01  mikbr212
 * despammed
 *
 * Revision 1.16  2001/07/29 13:45:34  mikbr212
 * never dribble after kickoff
 *
 * Revision 1.15  2001/07/29 13:30:43  mikbr212
 * new dribble
 *
 * Revision 1.14  2001/07/28 11:56:44  sigge
 * Output the debug messages to stderr.
 *
 * Revision 1.13  2001/07/27 14:36:38  mikbr212
 * Changed pass
 *
 * Revision 1.12  2001/07/27 09:37:06  mikbr212
 * Fixed compilation error
 *
 * Revision 1.11  2001/07/26 17:41:14  andbo928
 * Some pass stuff.
 *
 * Revision 1.10  2001/07/16 15:36:47  andbo928
 * Now uses surescore.
 *
 * Revision 1.9  2001/07/12 12:50:27  andbo928
 * Cosmetic changes.
 *
 * Revision 1.8  2001/07/11 16:49:37  sigge
 * New AdvanceBall with dribble and stuff.
 *
 * Revision 1.7  2001/07/06 14:31:41  sigge
 * uSucks -> uSuck
 *
 * Revision 1.6  2001/07/06 14:13:18  sigge
 * New model/utility stuff
 *
 * Revision 1.5  2001/07/06 13:55:20  sigge
 * Basic AdvanceBall with new model/utility stuff
 *
 * Revision 1.4  2001/07/03 15:24:18  andbo928
 * Added utility methods.
 *
 * Revision 1.3  2001/07/03 13:23:40  andbo928
 * Cheezy stub that kicks the ball towards their goal.
 *
 * Revision 1.2  2001/07/02 15:59:14  andbo928
 * Cosmetic changes.
 *
 * Revision 1.1  2001/06/19 21:35:10  mikbr212
 * Created stubs
 *
 * Revision 1.1  2001/06/19 14:11:43  mikbr212
 * Created
 *
 */

/**
 * @file AdvanceBallIntention.cc
 * Implementation file for the class AdvanceBallIntention.
 *
 * @par Unit:
 *      Framework
 * $Revision: 1.19 $
 * $Author: mikbr212 $
 * $Date: 2001/08/01 14:17:00 $
 */

#include "AdvanceBallIntention.h"
#include "ClearBall.h"
#include "Dribble.h"
#include "FuzzyScore.h"
#include "GameView.h"
#include "Pass.h"
#include "Role.h"
#include "SureScore.h"

RS_BEGIN_NAMESPACE

/** Constructs an intention.
 */
AdvanceBallIntention::AdvanceBallIntention()
  : Intention()
{
}

/** Destroys the intention.
 */
AdvanceBallIntention::~AdvanceBallIntention() {
}

// Uncomment either chooseSkill or generatePlan
/* Generates a plan.
 * Returns an empty plan.
 *
 Plan AdvanceBallIntention::generatePlan() {
 return Plan(); // Your plan
 }
*/

// Uncomment either chooseSkill or generatePlan
/* Chooses a skill. It works like a mini-decision maker.
 */
Skill* AdvanceBallIntention::chooseSkill() {
  static Dribble dribble;
  static ClearBall clearBall;
//    static FuzzyScore score;
  static SureScore score;
  static Pass pass;
  const AgentState& agent = worldStateView->getWorldState().getAgent();
  static Role role;

#define RED "[01;31m"
#define YELLOW "[01;33m"
#define GREEN "[01;32m"
#define BLUE "[01;34m"
#define FNORD "[01;35m"
#define NORMAL "[0m"

//  #define ADVANCE_BALL_DEBUG
#ifdef ADVANCE_BALL_DEBUG
#define ADEBUG(x) (cerr << x NORMAL << endl)
#else
#define ADEBUG(x)
#endif

  if(score.applicable().getValue() > 0.5)
  {
    ADEBUG(RED "Score!");
    return &score;
  }

  const float clearBallZone = -40.0;
  if(worldStateView->getWorldState().getAgent().getPosition().getX()
     < clearBallZone)
  {
    ADEBUG(YELLOW "Clearing(def)!");
    return &clearBall;
  }

  // Pass
  const PassTree& pt(passTreeView->getAgentPassTree());
  PassTree::Analysis analysis(pt.getAnalysis());
  if(analysis.hasPassTarget()) {
    PlayerState passTarget(analysis.getPassTarget());

    bool validTarget(true);
    if(passTarget.getC() < 0.3)
      validTarget = false;

    if(validTarget) {
      TVector goal(52.5,0);
      float passTargetToGoal = (goal - passTarget.getPosition()).getLength();
      float agentToGoal = (goal - agent.getPosition()).getLength();

      if(passTargetToGoal > agentToGoal + 3) { // back pass
	if(analysis.getBestScoreValue() <
	   analysis.getAgentScoreValue() + 0.2)
	  validTarget = false;
      }
    }

    pass.setVictim(analysis.getPassTarget());
    if(validTarget && pass.applicable().getValue() > 0.5) {
      ADEBUG(GREEN "Pass!");
      return &pass;
    }
  }

  if(statisticsView->getCyclesSinceOurKickoff() < 15) {
    // Hard coded pass from player 8 to player 9 then 11
    const WorldState& ws = worldStateView->getWorldState();
    const std::vector<PlayerState>& teammates = ws.getTeammates();
    int hasVictim = 0;
    PlayerState victim;
    for(int i=0; i<teammates.size(); i++)
    {
      if((agent.getUniformNumber() == 8
	  && teammates[i].getUniformNumber() == 9)
	 ||(agent.getUniformNumber() == 9
	    && teammates[i].getUniformNumber() == 11))
      {
	hasVictim = 1;
	victim = teammates[i];
      }
    }
    if(hasVictim)
    {
      pass.setVictim(victim);
      return &pass;
    }
    else
      return &clearBall;
  }

  // Dribble / get more info

//    if(statisticsView->getOpponentBallDistance() > 7.0)
//    {
//      ADEBUG(BLUE "Dribble!");
//      return &dribble;
//    }

// Dribble / get more info
  if(!gameView->isOurBall()
     && role.getRoleType() != Role::rtDefensive)
  {
    dribble.setSpeed(1.0);
    dribble.setSafeCycles(20);
    if(dribble.applicable() > FuzzyBool(0.5))
    {
      ADEBUG(BLUE "Dribble 1.0!");
      return &dribble;
    }

    dribble.setSpeed(0.3);
    dribble.setSafeCycles(10);
    if(dribble.applicable() > FuzzyBool(0.5))
    {
      ADEBUG(BLUE "Dribble 0.3!");
      return &dribble;
    }

    dribble.setSpeed(0.0);
    dribble.setSafeCycles(6);
    if(dribble.applicable() > FuzzyBool(0.5))
    {
      ADEBUG(BLUE "Dribble 0.0!");
      return &dribble;
    }
  }

  ADEBUG(YELLOW "Clear!");
  return &clearBall;
}

/** Models a player.
 * This intention simply calls plr.doNull().
 * @param plr the player to be modelled
 */
void AdvanceBallIntention::model(WorldState& ) {
  // FIXME: Proper modelling should be done
}

/** Checks if the utility of this intention.
 */
Intention::Utility AdvanceBallIntention::utility()
{
  static ClearBall clearBall;
  static FuzzyScore score;
  static Pass pass;

  const WorldState& worldState = worldStateView->getWorldState();
  if(worldState.getAgent().isBallKickable(worldState.getBall()))
    {
    if(score.applicable().getValue() > 0.7)
      return Intention::uVeryGood;
    if(score.applicable().getValue() > 0.5)
      return Intention::uGood;

    if(pass.applicable().getValue() > 0.5)
      return Intention::uGood;

    return Intention::uNormal;
  }
  else
    return Intention::uSuck;
}

Intention::Utility AdvanceBallIntention::modelUtility(const WorldState& ws)
{
  // FIXME: Proper modelling should be done
  static ClearBall clearBall;
  static FuzzyScore score;
  static Pass pass;

  if(ws.getAgent().isBallKickable(ws.getBall()))
  {
    if(score.applicable().getValue() > 0.7)
      return Intention::uVeryGood;
    if(score.applicable().getValue() > 0.5)
      return Intention::uGood;

    if(pass.applicable().getValue() > 0.5)
      return Intention::uGood;
    return Intention::uNormal;
  }
  else
    return Intention::uSuck;
}

/** Returns the persistence of this intention.
 * This intention is very persistent.
 * @return a float between zero (not persistent) and one (very persistent)
 */
float AdvanceBallIntention::getPersistence() const {
  return 1.0; // Very persistent
}

/** Returns the ID that identifies this (type of) intention.
 */
IntentionId AdvanceBallIntention::getID() const {
  return ADVANCE_BALL_INTENTION_ID;
}

RS_END_NAMESPACE

/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "AdvanceBallIntention.icc"
#endif
