/* -*- Mode: C++ -*-
 * $Id: AdvanceBallIntention.h,v 1.5 2001/07/06 14:13:18 sigge Exp $
 *
 * IDENTIFICATION
 *
 * $Revision: 1.5 $
 * Created by: Mikael Brnnstrm, Andreas Bckert, Andreas Sigfridsson
 * Last modified by $Author: sigge $ $Date: 2001/07/06 14:13:18 $
 *
 *
 * HISTORY
 *
 * $Log: AdvanceBallIntention.h,v $
 * Revision 1.5  2001/07/06 14:13:18  sigge
 * New model/utility stuff
 *
 * Revision 1.4  2001/07/06 13:55:20  sigge
 * Basic AdvanceBall with new model/utility stuff
 *
 * Revision 1.3  2001/07/03 15:24:18  andbo928
 * Added utility methods.
 *
 * Revision 1.2  2001/07/03 13:23:40  andbo928
 * Cheezy stub that kicks the ball towards their goal.
 *
 * Revision 1.1  2001/06/19 21:35:10  mikbr212
 * Created stubs
 *
 * Revision 1.1  2001/06/19 14:11:43  mikbr212
 * Created
 *
 */

/**
 * @file Intention.h
 * Definition file for the class Intention.
 *
 * @par Unit:
 *      Framework
 * $Revision: 1.5 $
 * $Author: sigge $
 * $Date: 2001/07/06 14:13:18 $
 */

#ifndef ADVANCE_BALL_INTENTION_H
#define ADVANCE_BALL_INTENTION_H

#include "Intention.h"

RS_BEGIN_NAMESPACE

/** 
 * @class AdvanceBallIntention AdvanceBallIntention.h
 * This intention is used when the player wants to advance
 * the ball by dribbling it towards the goal or passing
 * it to another player that is more strategically placed.
 *
 * $Revision: 1.5 $
 * $Author: sigge $
 * $Date: 2001/07/06 14:13:18 $
 */
class AdvanceBallIntention : public Intention {
public:
  AdvanceBallIntention();
  virtual ~AdvanceBallIntention();

  // Uncomment either chooseSkill or generatePlan
  //  virtual Plan generatePlan();
  virtual void model(WorldState&);

  // Uncomment if the agent should be threated specially
  //virtual Utility utility();
  virtual Utility utility();
  virtual Utility modelUtility(const WorldState&);

  virtual float getPersistence() const;

  virtual IntentionId getID() const;

protected:
  // Uncomment either chooseSkill or generatePlan
  virtual Skill* chooseSkill();

  // This method should not need to be overloaded
  //  virtual Plan getPlan(bool clear=false);

  // The following method is inherited:
  //  virtual Plan getPlan(bool clear=false);
  
  // The following views are inherited:
  //    ParameterView* parameterView;
  //    WorldStateView* worldStateView;
  //    PassTreeView* passTreeView;
  //    GameView* gameView;
  
private:
  /** Hide copy-constructor. */
  AdvanceBallIntention(const AdvanceBallIntention&);
  /** Hide assignment operator. */
  AdvanceBallIntention& operator= (const AdvanceBallIntention&);
};

RS_END_NAMESPACE

/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "AdvanceBallIntention.icc"
#endif

#endif // ADVANCE_BALL_INTENTION_H
