/* -*- Mode: C++ -*-
 * $Id: AssistOffensiveIntention.h,v 1.6 2001/07/29 09:43:30 mikbr212 Exp $
 *
 * IDENTIFICATION
 *
 * $Revision: 1.6 $
 * Created by: Mikael Brnnstrm, Andreas Bckert, Andreas Sigfridsson
 * Last modified by $Author: mikbr212 $ $Date: 2001/07/29 09:43:30 $
 *
 *
 * HISTORY
 *
 * $Log: AssistOffensiveIntention.h,v $
 * Revision 1.6  2001/07/29 09:43:30  mikbr212
 * Adjusted some things
 *
 * Revision 1.5  2001/07/06 20:48:27  mikbr212
 * Added some reasonable utilities
 *
 * Revision 1.4  2001/07/06 14:39:20  mikbr212
 * fixed utility
 *
 * Revision 1.3  2001/07/06 08:30:53  mikbr212
 * Made the intention work (but not yet in the new design)
 *
 * Revision 1.2  2001/07/03 15:24:18  andbo928
 * Added utility methods.
 *
 * Revision 1.1  2001/06/19 21:35:10  mikbr212
 * Created stubs
 *
 * Revision 1.1  2001/06/19 14:11:43  mikbr212
 * Created
 *
 */

/**
 * @file Intention.h
 * Definition file for the class Intention.
 *
 * @par Unit:
 *      Framework
 * $Revision: 1.6 $
 * $Author: mikbr212 $
 * $Date: 2001/07/29 09:43:30 $
 */

#ifndef ASSIST_OFFENSIVE_INTENTION_H
#define ASSIST_OFFENSIVE_INTENTION_H

#include "Intention.h"

RS_BEGIN_NAMESPACE

/** 
 * @class AssistOffensiveIntention AssistOffensiveIntention.h
 * This intention is applicable for a non-defensive player
 * that is not taking part in the offensive, but should do.
 * This could be a player that is slightly below the teammate with the
 * ball.
 *
 * $Revision: 1.6 $
 * $Author: mikbr212 $
 * $Date: 2001/07/29 09:43:30 $
 */
class AssistOffensiveIntention : public Intention {
public:
  AssistOffensiveIntention();
  virtual ~AssistOffensiveIntention();

  virtual Intention::Utility utility();
  virtual Intention::Utility modelUtility(const WorldState&);

  virtual void model(WorldState&);

  virtual float getPersistence() const;

  virtual IntentionId getID() const;
  
protected:
  TVector getPosition();
  Utility getNeedOfAssistance(const WorldState& ws);
  virtual Skill* chooseSkill();
  bool isSpecialSituation();
  Utility commonUtility(const WorldState&);
  // The following views are inherited:
  //    ParameterView* parameterView;
  //    WorldStateView* worldStateView;
  //    PassTreeView* passTreeView;
  //    GameView* gameView;

private:
  /** Hide copy-constructor. */
  AssistOffensiveIntention(const AssistOffensiveIntention&);
  /** Hide assignment operator. */
  AssistOffensiveIntention& operator= (const AssistOffensiveIntention&);
};

RS_END_NAMESPACE

/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "AssistOffensiveIntention.icc"
#endif

#endif // ASSIST_OFFENSIVE_INTENTION_H
