/* -*- Mode: C++ -*-
 * $Id: ContinueIntention.h,v 1.4 2001/07/26 18:58:49 mikbr212 Exp $
 *
 * IDENTIFICATION
 *
 * $Revision: 1.4 $
 * Created by: Mikael Brnnstrm, Andreas Bckert, Andreas Sigfridsson
 * Last modified by $Author: mikbr212 $ $Date: 2001/07/26 18:58:49 $
 *
 *
 * HISTORY
 *
 * $Log: ContinueIntention.h,v $
 * Revision 1.4  2001/07/26 18:58:49  mikbr212
 * Changed to new intention design
 *
 * Revision 1.3  2001/07/03 15:24:18  andbo928
 * Added utility methods.
 *
 * Revision 1.2  2001/06/19 21:34:43  mikbr212
 * Added comments
 *
 * Revision 1.1  2001/06/19 14:11:43  mikbr212
 * Created
 *
 */

/**
 * @file Intention.h
 * Definition file for the class Intention.
 *
 * @par Unit:
 *      Framework
 * $Revision: 1.4 $
 * $Author: mikbr212 $
 * $Date: 2001/07/26 18:58:49 $
 */

#ifndef CONTINUE_INTENTION_H
#define CONTINUE_INTENTION_H

#include "Intention.h"

RS_BEGIN_NAMESPACE

/** 
 * @class ContinueIntention ContinueIntention.h
 * This intention is only used for modelling purposes.
 * It will make the player continue with constant speed
 * in the direction and with the speed it currently has.
 *
 * $Revision: 1.4 $
 * $Author: mikbr212 $
 * $Date: 2001/07/26 18:58:49 $
 */
class ContinueIntention : public Intention {
public:
  ContinueIntention();
  virtual ~ContinueIntention();

  virtual void model(WorldState&);
  virtual Utility modelUtility(const WorldState&);

  virtual Utility utility();
  virtual Plan generatePlan();

  // Uncomment if the agent should be threated specially
  //virtual Utility utility();
  virtual float getPersistence() const;

  virtual IntentionId getID() const;
  
protected:
  // The following views are inherited:
  //    ParameterView* parameterView;
  //    WorldStateView* worldStateView;
  //    PassTreeView* passTreeView;
  //    GameView* gameView;
  
private:
  /** Hide copy-constructor. */
  ContinueIntention(const ContinueIntention&);
  /** Hide assignment operator. */
  ContinueIntention& operator= (const ContinueIntention&);
};

RS_END_NAMESPACE


#endif // CONTINUE_INTENTION_H

