#include "Operation.h"
#include "GenePool.h"
#include "ControlOps.h"
#include <iterator>

RS_BEGIN_NAMESPACE

GenePool::GenePool()
{
  genes.clear();
  utility.clear();
  numOffsprings.clear();
}
GenePool::~GenePool()
{
  for(unsigned int i = 0; i < genes.size(); i++)
    delete genes[i];
  genes.clear();
  utility.clear();
  numOffsprings.clear();
}

int GenePool::getRandomIndex()
{
  // Do some random selections.
  float utilSum = 0.0;
  for(unsigned int i = 0; i < utility.size(); i++)
    {
      utilSum += utility[i];
    }

  float selectPoint = (float)rand() * utilSum / (float)RAND_MAX;
  int selectIndex = 0;
  for(unsigned int i = 0; i < utility.size(); i++)
    {
      selectPoint -= utility[i];
      if(selectPoint < 0) {
	selectIndex = i;
	break;
      }
    }
  return selectIndex;
}

bool GenePool::isEmpty()
{
  if(utility.size() == 0)
    return true;
  return false;
}

Operation* GenePool::getGene()
{
  // Do some random selections...
  int index1, index2;
  index1 = getRandomIndex();
  index2 = getRandomIndex();
  numOffsprings[index1] = numOffsprings[index1] + 1;
  numOffsprings[index2] = numOffsprings[index2] + 1;

  return spliceGenes(genes[index1], genes[index2]);
}

void GenePool::addGene(float util, Operation* gene)
{
  utility.insert(utility.end(), util);  
  genes.insert(genes.end(), gene->clone());
  numOffsprings.insert(numOffsprings.end(), 0);
  decimate();
}
void GenePool::decimate()
{
  // Reduce the pool to genes within a 50% limit of most useful gene.

  //  #define decimate_threshold 0.05
  #define decimate_threshold 0.001
 // Reduce max number of offsprings to avoid inbreedings.
  #define maxOffsprings 10
  // We want a pool of atleast 500.
  if(utility.size() < 100)
    return;


  cerr << "DECIMATING GENE POOL - PRE: " << utility.size() << "," << genes.size();

  float utilMax = -1;
  for(unsigned int i = 0; i < utility.size(); i++)
    {
      if (utility[i] > utilMax) {
	utilMax = utility[i];
      }
    }
  
  std::vector<float> keptUtils;
  std::vector<Operation*> keptGenes;
  std::vector<int> keptOffsprings;

  int num = 0;

  for(unsigned int i = 0; i < utility.size(); i++)
    {
      if(utility[i] < utilMax * decimate_threshold
	 || numOffsprings[i] > maxOffsprings)

	{
	  delete genes[i];
	}
      else
	{
	  keptUtils.insert(keptUtils.end(), utility[i]);
	  keptGenes.insert(keptGenes.end(), genes[i]);
	  keptOffsprings.insert(keptOffsprings.end(), numOffsprings[i]);
	}
    }

  utility.clear();
  genes.clear();
  numOffsprings.clear();
  utility = keptUtils;
  genes = keptGenes;
  numOffsprings = keptOffsprings;

  cerr << " - POST: " << utility.size() << "," << genes.size() << endl;

}

int GenePool::countOperations(Operation *op)
{
  std::vector<Operation*> args = op->getArguments();
  int res = 1; // This operation.
  for(unsigned int i = 0; i<args.size(); i++)
    res += countOperations(args[i]);

  return res;
}

std::vector<Operation*> GenePool::getOperations(Operation *op, opType type)
{
  std::vector<Operation*> args = op->getArguments();
  std::vector<Operation*> res;

  if (op->getReturnType() == type)
    {
      res.insert(res.begin(), op);
    }

  for(unsigned int i = 0; i<args.size(); i++) {
    std::vector<Operation*> subRes = getOperations(args[i], type);
    res.insert(res.end(), subRes.begin(), subRes.end());
  }

  return res;
}

std::vector<Operation*> GenePool::getOperations(Operation *op)
{
  // flattens the gene "tree"

  std::vector<Operation*> fltRes = getOperations(op, opFloat);
  std::vector<Operation*> pntRes = getOperations(op, opPoint);
  std::vector<Operation*> strRes = getOperations(op, opString);
  fltRes.insert(fltRes.end(), pntRes.begin(), pntRes.end());
  fltRes.insert(fltRes.end(), strRes.begin(), strRes.end());
  return fltRes;
}

Operation* GenePool::spliceGenes(Operation *gene1, Operation* gene2)
{
//    cout << " SPLICING " << endl;
//    gene1->print();
//    cout << " WITH " << endl;
//    gene2->print();

  // Splice to genes together.
  
  Operation *dupGene1 = gene1->clone();

  // Mutations!
  if( (rand() % 100) < 10 )
    {
      Operation *old = dupGene1;
      dupGene1 = mutate(old);
      //      delete old;
    }

  // Select input point of gene1.
  std::vector<Operation*> ops1 = getOperations(dupGene1);

  // This shouldn't happen.
  if(ops1.size() == 0) {
    cout << "GenePool::spliceGenes - THIS SHOULDN'T HAPPEN!" << endl;
  }

  int inputIndex = rand() % (ops1.size());

  Operation* inputOp = ops1[inputIndex];
  std::vector<Operation*> inputArgs  = inputOp->getArguments();
  
  // Select a random argument as input point.
  int numArgs = inputArgs.size();

  // if we don't have any input arguments to this
  // operation we let things be.
  if(numArgs == 0)
    return dupGene1;

  int inputArg = rand() % numArgs;
  

//    cout << " SPLICE OUT " << endl;
//    inputArgs[inputArg]->print();

  // Find a suitable input operation.

  std::vector<Operation*> ops2 = getOperations(gene2, inputOp->getArgumentTypes()[inputArg]);

  // If no suitable input operations are found we let things be.
  if(ops2.size() == 0)
    return dupGene1;

  // Select a operation at random.
  int argIndex = rand() % ops2.size();
  Operation* argOp = ops2[argIndex];

//    cout << " SPLICE IN " << endl;
//    argOp->print();

  // Mutations!
  if( (rand() % 100) < 10 )
    {
      Operation *old = argOp;
      argOp = mutate(old);
    }

  inputArgs[inputArg] = argOp;
  inputOp->setArguments(inputArgs);

  return dupGene1;
}

Operation* GenePool::setRandomArguments(Operation *o)
{
  static OpFloatConstant cFloat;
  static OpPointConstant cPoint;
  static OpTurn turn;

  std::vector<Operation*> args;
  std::vector<opType> argTypes = o->getArgumentTypes();

  for(unsigned int i = 0; i < argTypes.size(); i++)
    {
      switch(argTypes[i])
	{
	case opFloat:
	  cFloat = OpFloatConstant(randomFloat());
	  args.insert(args.end(), &cFloat);
	  break;
	case opPoint:
	  cPoint = OpPointConstant(randomPoint());
	  args.insert(args.end(), &cPoint);
	  break;
	case opString:
	  std::vector<Operation*> tmp;
	  tmp.insert(tmp.end(),  &cFloat );
	  turn.setArguments(tmp);
	  args.insert(args.end(), &turn);
	  break;

	}
    }

  o->setArguments(args);
  return o;
}

float GenePool::randomFloat()
{
  return (float)rand() * 100.0 / (float)RAND_MAX;
}

Point GenePool::randomPoint()
{
  return Point(randomFloat(), randomFloat());
}

std::string GenePool::randomString()
{
  return "";
}

Operation* GenePool::getRandomFloatGene()
{

//  FloatMath.h
//  class OpFloatPlus : public FloatOperation
//  class OpFloatMinus : public FloatOperation
//  class OpFloatMultiply : public FloatOperation
//  class OpFloatDivide : public FloatOperation
//  class OpFloatConstant : public FloatOperation
//  class OpFloatArg : public FloatOperation

//  PointMath.h
//  class OpPointMultiply : public FloatOperation
//  class OpPointLength : public FloatOperation
//  class OpPointAngle : public FloatOperation
//  class OpPointX : public FloatOperation
//  class OpPointY : public FloatOperation

//  ObjMath.h
//  class OpAgentDir : public FloatOperation

//  ControlOps.h
//  class OpFloatIf : public FloatOperation

  // Total 13
  int rndIndex = rand() % 13;
  Operation *o;
  switch(rndIndex)
    {
    case 0:
      o = new OpFloatPlus();
      break;
    case 1:
      o = new OpFloatMinus();
      break;
    case 2:
      o = new OpFloatMultiply();
      break;
    case 3:
      o = new OpFloatDivide();
      break;
    case 4:
      o = new OpFloatArg();
      break;
    case 5:
      o = new OpPointMultiply();
      break;
    case 6:
      o = new OpPointLength();
      break;
    case 7:
      o = new OpPointAngle();
      break;
    case 8:
      o = new OpPointX();
      break;
    case 9:
      o = new OpPointY();
      break;
    case 10:
      o = new OpAgentDir();
      break;
    case 11:
      o = new OpFloatConstant(10.0);
      break;
    case 12:
      o = new OpFloatIf();
      break;
    default:
      o = new OpFloatConstant(0.0);
      break;
    }

  return o;

}

Operation* GenePool::getRandomStringGene()
{

//  CommandOps.h
//  class OpKick : public StringOperation
//  class OpDash : public StringOperation
//  class OpTurn : public StringOperation

//  StringOps.h
//  class OpStringPlus : public StringOperation

//  ControlOps.h
//  class OpStringIf : public StringOperation

  // Total 5
  int rndIndex = rand() % 5;
  Operation *o;
  switch(rndIndex)
    {
    case 0:
      o = new OpKick();
      break;
    case 1:
      o = new OpDash();
      break;
    case 2:
      o = new OpTurn();
      break;
    case 3:
      o = new OpStringIf();
      break;
    case 4:
    default:
      o = new OpStringPlus();
      break;
    }
  return o;
}

Operation* GenePool::getRandomPointGene()
{

//  PointMath.h
//  class OpPointPlus : public PointOperation
//  class OpPointMinus : public PointOperation
//  class OpPointScale : public PointOperation
//  OpPointConstant()
//  class OpPolPoint : public PointOperation
//  class OpPointArg : public PointOperation
//  class OpPoint : public PointOperation

//  ObjMath.h
//  class OpAgentPos : public PointOperation
//  class OpAgentSpeed : public PointOperation

//  ControlOps.h
//  class OpPointIf : public PointOperation


  // Total 10
  int rndIndex = rand() % 10;
  Operation *o;
  switch(rndIndex)
    {
    case 0:
      o = new OpPointPlus();
      break;
    case 1:
      o = new OpPointMinus();
      break;
    case 2:
      o = new OpPointScale();
      break;
    case 3:
      o = new OpPointConstant(randomPoint());
      break;
    case 4:
      o = new OpPolPoint();
      break;
    case 5:
      o = new OpPointArg();
      break;
    case 6:
      o = new OpPoint();
      break;
    case 7:
      o = new OpAgentPos();
      break;
    case 8:
      o = new OpAgentSpeed();
      break;
    case 10:
      o = new OpPointIf();
      break;
    default:
      o = new OpPointConstant(randomPoint());
      break;
    }

  return o;
}

Operation* GenePool::mutate(Operation *gene)
{
  // Mutate a gene.
  cout << "MUTATING IN: " << endl;
  gene->print();
  Operation* o;

  switch(gene->getReturnType())
    {
    case opFloat:
      o = getRandomFloatGene();
      break;
    case opPoint:
      o = getRandomPointGene();
      break;
    case opString:
      o = getRandomStringGene();
      break;
    default:
      cerr << "THIS SHOULDN'T HAPPEN!" << endl;
      break;
    }

  o = setRandomArguments(o);
  cout << "MUTATING OUT: " << endl;
  o->print();

  return o;
}


RS_END_NAMESPACE









