/** * @file GeneSkill.cc
 * Implementation of the GeneSkill skill.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.1.1.1 $
 * $Author: sigge $
 * $Date: 2001/04/08 11:33:58 $
 */

#include "GeneSkill.h"
#include "KickCommand.h"
#include "DashCommand.h"
#include "TurnCommand.h"
#include "KickBallToSpeedVector.h"

#define KICK_RANGE 30
#define LOOK_RANGE 20
#define SAFE_MARGIN 2
#define CB_FRIENDLY 0.2
#define CB_OPPONENT -1.0
#define CB_ADVANCE  0.5
#define CB_PRECISION 0.0

RS_BEGIN_NAMESPACE

// --- CONSTRUCTORS ---

/**
 * Constructor.
 */

GeneSkill::GeneSkill()
  : Skill(),
    parameterView(NULL), //,
    markerView(NULL),
    agentView(NULL),
    ballView(NULL),
    playerView(NULL),
    kickBall(Vector())
{
  getView(parameterView, PARAMETER_VIEW_ID);
  getView(playerView, FUZZY_PLAYER_VIEW_ID);
  getView(markerView, MARKER_VIEW_ID, parameterView);
  getView(agentView, AGENT_VIEW_ID, parameterView);
  getView(ballView, FUZZY_BALL_VIEW_ID);
  kickBall.setFastKick(TRUE);
}

// --- DESTRUCTOR ---

GeneSkill::~GeneSkill()
{
  viewManager->releaseView(PARAMETER_VIEW_ID);
  //  viewManager->releaseView(MARKER_VIEW_ID);
  viewManager->releaseView(AGENT_VIEW_ID);
  viewManager->releaseView(BALL_VIEW_ID);
  viewManager->releaseView(PLAYER_VIEW_ID);
}

// --- METHODS ---

/**
 * Genrates a plan for achieving a pass.
 * Chooses the closest of the passvictims if the argument is
 * supplied, otherwise pass the best player according to 
 * TerritoryControlView
 */

std::string GeneSkill::trim(std::string str)
{
 // remove leading spaces.
  for(unsigned int i = 0; i < str.size(); i++) {
    if(str[i] != ' ') {
      //      cout << "trim: " << str.substr(i, str.size() - i) << endl;
      return str.substr(i, str.size() - i);
    }
  }
  return "";
}

std::string GeneSkill::first(std::string str)
{
  str = trim(str);
  for(unsigned int i = 0; i < str.size(); i++) {
    if(str[i] == ' ') {
      //      cout << "first: " << str.substr(0,i) << endl;
      return str.substr(0,i);
    }
  }
  return str;
}

std::string GeneSkill::rest(std::string str)
{
  str = trim(str);
  for(unsigned int i = 0; i < str.size(); i++) {
    if(str[i] == ' ') {
      //      cout << "rest: " << str.substr(i, str.size() - i) << endl;
      return str.substr(i, str.size() - i);
    }
  }
  return "";
}

Plan GeneSkill::generatePlan()
{
//    if(gene == NULL)
//      return Plan();

  std::string res = gene->getString();
  //  std::string res = "turn 90 dash 100 kick 10 20";
  std::string str = res;

  // Parse string for commands.
  Plan p;
  Command *cmd;

  //  cout << "Gene parsing: " << endl;


  while(str.size() > 1)
    {

      str = trim(str);
      std::string cmdStr = first(str);
      str = rest(str);

      if(cmdStr == "kick") {
	std::string arg1 = first(str);
	str = rest(str);
	std::string arg2 = first(str);
	str = rest(str);

	cmd = new KickCommand(atof(arg1.c_str()), atof(arg2.c_str()));
	//	cout << cmd->getDescription() << endl;

	p.addStep(Step(cmd));
	delete cmd;
      }

      if(cmdStr == "dash") {
	std::string arg = first(str);
	str = rest(str);

	cmd = new DashCommand(atof(arg.c_str()));
	//	cout << cmd->getDescription() << endl;

	p.addStep(Step(cmd));
	delete cmd;
      }
      
      if(cmdStr == "turn") {
	std::string arg = first(str);
	str = rest(str);

	cmd = new TurnCommand(atof(arg.c_str()));
	//	cout << cmd->getDescription() << endl;

	p.addStep(Step(cmd));
	delete cmd;
      }
    }

  return p;
}


/**
 * Persistent.
 * FALSE
 */
FuzzyBool GeneSkill::persistent()
{
  return FALSE;
}

/**
 * Returns the applicable value
 */

FuzzyBool GeneSkill::applicable()
{
  // Always true.. hehe...
  return TRUE;
}

void GeneSkill::setGene(Operation *gn)
{
  if(gn->getReturnType() != opString) {
    cout << "GeneSkill::setGene, only genes returning strings can be set!" << endl;
    return;
  }
  gene = gn;
}

RS_END_NAMESPACE








