/* -*- Mode: C++ -*-
 * $Id: Go.cc,v 1.25 2001/07/01 12:35:13 andbo928 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: Go.cc
 * Unitname: Framework/skill
 * $Revision: 1.25 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: andbo928 $ $Date: 2001/07/01 12:35:13 $
 *
 *
 * HISTORY
 *
 * $Log: Go.cc,v $
 * Revision 1.25  2001/07/01 12:35:13  andbo928
 * Now charges towards a position where the ball will be in a while.
 *
 * Revision 1.24  2001/04/25 01:05:23  mikbr212
 * adjusted intercept
 *
 * Revision 1.23  2001/04/24 23:46:23  mikbr212
 * adjusted intercept
 *
 * Revision 1.21  2001/04/24 23:38:19  mikbr212
 * adjusted intercept
 *
 * Revision 1.20  2001/04/24 23:04:33  sigge
 * Fixed 'offside bug'
 *
 * Revision 1.19  2001/04/24 21:47:59  mikbr212
 * Added margin on advanceline
 *
 * Revision 1.18  2001/04/24 21:40:21  mikbr212
 * Foo
 *
 * Revision 1.17  2001/04/24 21:27:27  mikbr212
 * ??
 *
 * Revision 1.16  2001/04/24 20:42:53  mikbr212
 * more offensive
 *
 * Revision 1.15  2001/04/24 19:53:59  mikbr212
 * more offensive
 *
 * Revision 1.14  2001/04/24 14:23:54  mikbr212
 * fixed bugg
 *
 * Revision 1.13  2001/04/24 14:01:42  mikbr212
 * fixed bugg
 *
 * Revision 1.12  2001/04/24 13:14:55  mikbr212
 * added turn when MoveToPosition is not applicable
 *
 * Revision 1.11  2001/04/24 12:30:47  mikbr212
 * fixed
 *
 * Revision 1.10  2001/04/23 15:10:55  mikbr212
 * added smart intercept
 *
 * Revision 1.9  2001/04/23 14:40:39  mikbr212
 * added mark
 *
 * Revision 1.8  2001/04/22 14:13:47  mikbr212
 * A little more defensive players, but they still dont go home... yet
 *
 * Revision 1.7  2001/04/22 12:16:38  mikbr212
 * removed kick-and-chase ball effect
 *
 * Revision 1.6  2001/04/21 16:27:12  mikbr212
 * fixed stuff
 *
 * Revision 1.5  2001/04/20 18:54:14  mikbr212
 * Reimplemented
 *
 * Revision 1.2  2001/04/13 18:15:44  andbo928
 * Now works without a lot of old skills.
 *
 * Revision 1.1.1.1  2001/04/08 11:33:58  sigge
 * Imported sources
 *
 * Revision 1.7  2000/11/07 18:10:44  mikbr212
 * changed persistent
 *
 * Revision 1.6  2000/11/01 16:38:58  mikbr212
 * fixed constructor
 *
 * Revision 1.5  2000/11/01 16:01:39  mikbr212
 * changed constructor
 *
 * Revision 1.4  2000/11/01 12:42:59  mikbr212
 * Added possibility to do a fast kick
 *
 * Revision 1.3  2000/10/30 18:32:45  mikbr212
 * added comments
 *
 */

/**
 * @file Go.cc
 * Implementation file for the skill class Go.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.25 $
 * $Author: andbo928 $
 * $Date: 2001/07/01 12:35:13 $
 */


#include "Go.h"

#include "Display.h"

RS_BEGIN_NAMESPACE




/**
 * Constructor.
 */

Go::Go()
  : Skill(),
    parameterView(NULL),
    gameView(NULL),
    passTreeView(NULL),
    worldStateView(NULL),
    lookView(NULL)
{
  init();
}

/**
 * Destructor.
 */
Go::~Go()
{
  viewManager->releaseView(PARAMETER_VIEW_ID);
  viewManager->releaseView(GAME_VIEW_ID);
  viewManager->releaseView(PASS_TREE_VIEW_ID);
  viewManager->releaseView(WORLD_STATE_VIEW_ID);
  viewManager->releaseView(LOOK_VIEW_ID);
}

void Go::init()
{
  getView(parameterView, PARAMETER_VIEW_ID);
  getView(gameView, GAME_VIEW_ID);
  getView(passTreeView, PASS_TREE_VIEW_ID);
  getView(worldStateView, WORLD_STATE_VIEW_ID);
  getView(lookView, LOOK_VIEW_ID);

  const int innerDirs = 8;
  const float innerRadius = 1;
  for(int i=0; i<innerDirs; i++) 
    treeMovements.push_back(TVector::TPolarVector(innerRadius, float(i)*360/float(innerDirs)));

  const int midDirs = 8;
  const float midRadius = 6;
  for(int i=0; i<midDirs; i++) 
    treeMovements.push_back(TVector::TPolarVector(midRadius, float(i)*360/float(midDirs)));

  const int outerDirs = 8;
  const float outerRadius = 15;
  for(int i=0; i<outerDirs; i++) 
    treeMovements.push_back(TVector::TPolarVector(outerRadius, float(i)*360/float(outerDirs)));

}


TVector Go::getTreePosition(const float minX, const float maxX) {
  PassTree pt(passTreeView->getAgentPassTree());
  return pt.getPosition(treeMovements, minX, maxX);
}

int Go::getClosestOpponent(const std::vector<PlayerState>& op,
			   const TVector& position,
			   float minDist) {
  int retVal = -1;
  int index = 0;
  float distance;
  std::vector<const PlayerState>::iterator it;
  for(it=op.begin(); it!=op.end(); it++) {
    distance = (it->getPosition() - position).getLength();
    if(distance < minDist) {
      minDist = distance;
      retVal = index;
    }
    index++;
  }
  return retVal;
}

int Go::countPlayersBehindLine(const std::vector<PlayerState>& plrs,
			       float line,
			       bool skipGoalie) {
  int count = 0;
  std::vector<const PlayerState>::iterator plr;
  for(plr = plrs.begin(); plr != plrs.end(); plr++) 
    if(plr->getPosition().getX() < line &&
       !(skipGoalie && plr->getUniformNumber() == 1))
      count++;
  return count;
}

bool Go::isBallGoingOutsideFieldAndAgentNotInBetween() {
  const AgentState& agent(worldStateView->getWorldState().getAgent());
  const BallState& ball(worldStateView->getWorldState().getBall());
  float scale = (1 - pow(parameterView->SP_ball_decay,16)) /
    (1-parameterView->SP_ball_decay);
  TVector newBallPos(ball.getPosition() + ball.getSpeed()*scale);
  if((agent.getPosition() - newBallPos).getLength() <
     (agent.getPosition() - ball.getPosition()).getLength())
    return false;

  if(newBallPos.getX() < -52.5 || newBallPos.getX() > 52.5 ||
     newBallPos.getY() < -34.0 || newBallPos.getY() > 34.0)
    return true;
  return false;
}

Plan Go::turnPlan() {
  const BallState& ball(worldStateView->getWorldState().getBall());
  AgentState agent(worldStateView->getWorldState().getAgent());

  AngleDeg ballAngle((ball.getPosition() - agent.getPosition()).getAngle());
  float angle = lookView->getAngle(ballAngle - 90, ballAngle + 90).getDeg();

  ballAngle -= agent.getDirection();
  agent.doActualTurn(ballAngle.getDeg());
  agent.doTurnNeckTo(angle);
  return Plan(agent.getStep());
}

Plan Go::generatePlan() {
  const WorldState& world(worldStateView->getWorldState());
  const BallState& ball(worldStateView->getWorldState().getBall());
  const AgentState& agent(worldStateView->getWorldState().getAgent());
  const PassTree& pt(passTreeView->getAgentPassTree());

  const std::vector<PlayerState>& op(worldStateView->getWorldState().getOpponents());
  const std::vector<PlayerState>& tm(worldStateView->getWorldState().getTeammates());

  const int A_OtherAction = 0;
  const int A_GotoBall = 1;
  const int A_InterceptBall = 2;

  // Is closest to ball
  int action = A_OtherAction;
  if(pt.isRoot()) {
    interceptSkill.setMaxSteps(20);

    // Check if any opponent also is close to the ball
    float distToBall = (agent.getPosition() - ball.getPosition()).getLength();
    float minDist = max(distToBall + 1, distToBall * 1.20F);
    bool opponentIsClose = (getClosestOpponent(op, ball.getPosition(), minDist) != -1);
    
    bool ballIsHeadedOut = isBallGoingOutsideFieldAndAgentNotInBetween();
    bool iKickedBallRecently = (world.getGameTime() - world.getLastKickCycle() < 5 &&
				world.getLastKickerTeam() == WorldState::ourTeam &&
				world.getLastKickerUniformNumber() == agent.getUniformNumber());
    if(!opponentIsClose) {
      // Check if I should take the ball (at all) or just let it go out from the field
      
      // Ball is headed out
      if(ballIsHeadedOut) {
	switch(world.getLastKickerTeam()) {
	case WorldState::theirTeam: 
	  action = A_OtherAction;
	  break;
	case WorldState::ourTeam: 
	  action = A_InterceptBall;
	  break;
	default: 
	  action = A_GotoBall;
	  break;
	}
      } else { // Ball is not headed out
	if(iKickedBallRecently)
	  action = A_OtherAction;
	else
	  action = A_GotoBall;
      }
    } else { // Opponent is close
      if(iKickedBallRecently)
	action = A_OtherAction;
      else
	action = A_InterceptBall;
    }
    
    if(action == A_OtherAction)
      action = A_GotoBall;
    
    if(ball.getSpeed().getLength() > 0.5)
      action = A_InterceptBall; 

    if(action == A_InterceptBall && interceptSkill.applicable()) {
      return interceptSkill.generatePlan();
    } else {
      TVector desPos = ball.getPosition() + ball.getSpeed() * 14.0;
      goSkill.setPosition(desPos);
      if(goSkill.applicable())
	return goSkill.generatePlan();
      else
	return turnPlan();
    }
  }

  // Is it offensive mode?
  bool offensiveMode = false;
  std::vector<PlayMode> pms(gameView->getPlayModes(NOW));
  std::vector<PlayMode>::iterator pm;
  for(pm = pms.begin(); pm != pms.end(); pm++) {
    switch(*pm) {
    case PM_OUR_OFFSIDE_KICK:
    case PM_OUR_KICK_IN:
    case PM_OUR_CORNER_KICK:
    case PM_GOAL_KICK:
      offensiveMode = true;
    default:
      break;
    }
  }

  if(!offensiveMode && worldStateView->getPossession() > 0.7) {
    offensiveMode = true;
  }
  
  // Can I find an opponent to mark?
  interceptSkill.setMaxSteps(20);
  if(!offensiveMode && markPlayerSkill.applicable()) { 
    if(ball.isHeadedTo(markPlayerSkill.getPlayer().getPosition(), 7.0, 20) &&
       interceptSkill.applicable())
      return interceptSkill.generatePlan();
    else
      return markPlayerSkill.generatePlan();
  }

  // is ball headed to me?
  interceptSkill.setMaxSteps(16);
  if(ball.isHeadedTo(agent.getPosition(), 7.0, 20) &&
     interceptSkill.applicable())
    return interceptSkill.generatePlan();
    
  bool goHome = false;
  int myTmCount = countPlayersBehindLine(tm,agent.getPosition().getX(),true);
  int myOpCount = countPlayersBehindLine(op,agent.getPosition().getX(),false);

//    int tmCount = countPlayersBehindLine(tm,0,true);
//    int opCount = countPlayersBehindLine(op,0,false);

  if(myTmCount < min(myOpCount, (offensiveMode ? 4 : 8)))
    goHome = true;
//    else if(tmCount < 

//    int optimumCount = minMax(2,min(opCount,myOpCount),8);
//    if(myCount < optimumCount && 
//       !(myCount < 2 && agent.getPosition().getX() < 0) &&
//       )
//      goHome = true;

  // Defence?
  float minX = -52.5;
  float maxX = pt.getOurAdvanceLine() - 0.5;
  if(goHome) 
    maxX = agent.getPosition().getX() - 0.8;
  else if(offensiveMode) {
    if(myTmCount > 8)
      minX = min(min((double)0,(double)agent.getPosition().getX() + 0.9), maxX - 1.0);
    else
      minX = min(min((double)0,(double)agent.getPosition().getX()), maxX - 1.0);
  }

  goSkill.setPosition(getTreePosition(minX, maxX));
  if(goSkill.applicable())
    return goSkill.generatePlan();
  
  return turnPlan();
}

FuzzyBool Go::persistent()
{
  return false;
}


/**
 * Checks if preconds are true.
 */
FuzzyBool Go::applicable() {
  return true;
}



RS_END_NAMESPACE


