/* -*- Mode: C++ -*-
 * $Id: Go.h,v 1.9 2001/04/24 21:27:27 mikbr212 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: Go.h
 * Unitname: Framework/skill
 * $Revision: 1.9 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: mikbr212 $ $Date: 2001/04/24 21:27:27 $
 *
 *
 * HISTORY
 * $Log: Go.h,v $
 * Revision 1.9  2001/04/24 21:27:27  mikbr212
 * ??
 *
 * Revision 1.8  2001/04/24 13:14:55  mikbr212
 * added turn when MoveToPosition is not applicable
 *
 * Revision 1.7  2001/04/24 12:30:47  mikbr212
 * fixed
 *
 * Revision 1.6  2001/04/23 15:10:55  mikbr212
 * added smart intercept
 *
 * Revision 1.5  2001/04/23 14:40:39  mikbr212
 * added mark
 *
 * Revision 1.4  2001/04/22 14:13:47  mikbr212
 * A little more defensive players, but they still dont go home... yet
 *
 * Revision 1.3  2001/04/20 18:54:14  mikbr212
 * Reimplemented
 *
 * Revision 1.2  2001/04/13 18:15:44  andbo928
 * Now works without a lot of old skills.
 *
 * Revision 1.1.1.1  2001/04/08 11:33:58  sigge
 * Imported sources
 *
 * Revision 1.5  2000/11/01 16:38:59  mikbr212
 * fixed constructor
 *
 * Revision 1.4  2000/11/01 16:01:40  mikbr212
 * changed constructor
 *
 * Revision 1.3  2000/11/01 12:43:00  mikbr212
 * Added possibility to do a fast kick
 *
 * Revision 1.2  2000/10/28 20:29:30  mikbr212
 * generatePlan now uses time stamps and phases; Fixed wrong-calculations in recalcDesPos; Finally: made a working (?) skill!
 *
 * Revision 1.1  2000/10/07 14:26:56  mikbr212
 * *** empty log message ***
 *
 * Revision 1.2  2000/09/27 14:29:59  mikbr212
 * fixed ballspeed bug
 *
 * Revision 1.1  2000/09/24 19:30:19  andbo928
 * Various skills.
 *
 * Revision 1.1  2000/09/05 21:12:02  frehe
 * #include rs_log.h
 *
 */

#ifndef GO_H
#define GO_H


#include "Skill.h"

#include "ParameterView.h"
#include "PassTreeView.h"
#include "GameView.h"
#include "WorldStateView.h"
#include "LookView.h"

#include "MoveToPosition.h"
#include "MarkPlayer.h"
#include "Intercept.h"
#include "Display.h"

#include "rs_log.h"
#include "view_constants.h"


RS_BEGIN_NAMESPACE

/**
 * @class Go Go.h
 * This class encapsulates a scoring skill.
 *
 * <!--
 * @pre The preconditions for the skill are...
 * @post The postconditions for the skill are...
 * @invariant The class invariant is...
 *
 * @todo This needs to be done...
 * -->
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.9 $
 * $Author: mikbr212 $
 * $Date: 2001/04/24 21:27:27 $
 */
class Go : public Skill {
public:
  Go();
  virtual ~Go();

  virtual Plan generatePlan();

  virtual FuzzyBool persistent();

  virtual FuzzyBool applicable();

protected:
  Plan turnPlan();
  TVector getTreePosition(const float minX = -52.5F, const float maxX = 52.5F);
  int getClosestOpponent(const std::vector<PlayerState>& op,
			 const TVector& position,
			 float minDist);
  int countPlayersBehindLine(const std::vector<PlayerState>& plrs,
			     float line,
			     bool skipGoalie);

  bool isBallGoingOutsideFieldAndAgentNotInBetween();

  bool getMarkPlayer(PlayerState& player,
		     const float maxDist,
		     const float maxX);
  MoveToPosition goSkill;
  Intercept interceptSkill;
  MarkPlayer markPlayerSkill;

  std::vector<TVector> treeMovements;

  ParameterView* parameterView;
  GameView* gameView;
  PassTreeView* passTreeView;
  WorldStateView* worldStateView;
  LookView* lookView;

private:
  void init();

};


RS_END_NAMESPACE


#endif // !GO_H
