/* -*- Mode: C++ -*-
 * $Id: GoalieIntention.cc,v 1.21 2001/07/31 17:10:19 andbo928 Exp $
 *
 * IDENTIFICATION
 *
 * $Revision: 1.21 $
 * Created by: Mikael Brnnstrm, Andreas Bckert, Andreas Sigfridsson
 * Last modified by $Author: andbo928 $ $Date: 2001/07/31 17:10:19 $
 *
 *
 * HISTORY
 *
 * $Log: GoalieIntention.cc,v $
 * Revision 1.21  2001/07/31 17:10:19  andbo928
 * Minor changes.
 *
 * Revision 1.20  2001/07/31 15:34:53  andbo928
 * Hopefully removed the insertSpan bug.
 *
 * Revision 1.17  2001/07/30 11:25:26  mikbr212
 * Not that agressive goalie (changed 0.7 to 0.9)
 *
 * Revision 1.16  2001/07/29 14:19:09  andbo928
 * Doesn't exit penalty zone. More agressive behaviour.
 *
 * Revision 1.15  2001/07/28 15:27:57  andbo928
 * Now uses the new (and improved!) ClearBall.
 *
 * Revision 1.14  2001/07/28 07:18:12  mikbr212
 * Removed warnings!!
 *
 * Revision 1.13  2001/07/26 15:43:42  mikbr212
 * Fixed some includes
 *
 * Revision 1.12  2001/07/16 15:35:08  sigge
 * Threat forwarding implemented.
 *
 * Revision 1.11  2001/07/12 12:52:51  andbo928
 * Uses the new intercept skill.
 *
 * Revision 1.10  2001/07/09 14:47:33  andbo928
 * New code, now covers the largest uncoverd span in the goal.
 *
 * Revision 1.9  2001/07/06 14:46:43  andbo928
 * Added simple modelling.
 *
 * Revision 1.8  2001/07/06 14:31:41  sigge
 * uSucks -> uSuck
 *
 * Revision 1.7  2001/07/06 14:07:11  andbo928
 * Added modelutility
 *
 * Revision 1.6  2001/07/06 14:03:09  andbo928
 * Added modelutility
 *
 * Revision 1.5  2001/07/06 12:39:00  andbo928
 * Zonked applicable.
 *
 * Revision 1.4  2001/07/06 12:38:11  andbo928
 * Added utility comment and uSucks util value.
 *
 * Revision 1.3  2001/07/03 15:24:18  andbo928
 * Added utility methods.
 *
 * Revision 1.2  2001/07/03 13:15:29  sigge
 * GoalieIntention now uses the old goalie behaviour.
 *
 * Revision 1.1  2001/06/19 21:35:10  mikbr212
 * Created stubs
 *
 * Revision 1.1  2001/06/19 14:11:43  mikbr212
 * Created
 *
 */

/**
 * @file GoalieIntention.cc
 * Implementation file for the class GoalieIntention.
 *
 * @par Unit:
 *      Framework
 * $Revision: 1.21 $
 * $Author: andbo928 $
 * $Date: 2001/07/31 17:10:19 $
 */


#include "GoalieIntention.h"
#include "FindBall.h"
#include "ClearBall.h"
#include "FuzzyTeleport.h"
#include "CatchBall.h"
#include "MarkPosition.h"
#include "MoveToPosition.h"
#include "Intercept.h"
#include "InPenaltyArea.h"
#include "Display.h"

RS_BEGIN_NAMESPACE

/** Constructs an intention. 
 */
GoalieIntention::GoalieIntention()
  : Intention()
{
}

/** Destroys the intention. 
 */
GoalieIntention::~GoalieIntention()
{
}

/** Calculates which spans are currently uncoverd.
 */
void GoalieIntention::insertSpan(Span s, list<GoalieIntention::Span>& spans)
{
  if(s.start < 1.0 && s.end > 0.0)
    {
      list<GoalieIntention::Span>::iterator spanIt;
      for(spanIt = spans.begin(); spanIt != spans.end(); spanIt++)
      {
	// Check if we're done.
	if(s.start > spanIt->end)
	{
	  break; // We're done.
	}
	else
	{
	  if(s.end > spanIt->start)
	  {
	    if(s.start < spanIt->start
	       && s.end > spanIt->end)
	    {
	      // Remove the old span.
//  	      cout << "Remove span." << endl;
	      spanIt = spans.erase(spanIt);
	    }
	    else if(s.start < spanIt->start)
	    {
	      // Cut of start piece.
//  	      cout << "Trim start." << endl;
	      spanIt->start = s.end;
	    }
	    else if(s.end > spanIt->end)
	    {
	      // Cut of end piece;
//  	      cout << "Trim end." << endl;
	      spanIt->end = s.start;
	    }
	    else
	    {
	      // Split span into subspans.
//  	      cout << "Split span." << endl;
	      GoalieIntention::Span newSpan;
	      newSpan.start = spanIt->start;
	      newSpan.end = s.start;
	      spanIt->start = s.end;
	      spanIt = spans.insert(spanIt, newSpan);
	    }
	  }
	}
      }
    }
}

void GoalieIntention::calculateSpans(TVector threat, 
				     TVector& spanStart,
				     TVector& spanEnd)
{
  TLine goalLine = 
    TLine(TVector(-52.5, -1.05*parameterView->SP_goal_width/2.0),
	  TVector(-52.5, 1.05*parameterView->SP_goal_width/2.0));

  list<GoalieIntention::Span> spans;
  GoalieIntention::Span init;
  init.start = 0.0;
  init.end = 1.0;
  spans.push_back(init);

  const vector<PlayerState>& tm =
    worldStateView->getWorldState().getTeammates();

  // Assume that the agent is the goalie, for now...
  vector<PlayerState>::const_iterator it;
  for(it = tm.begin(); it != tm.end(); it++)
  {
    // Magic computation of the point where the extension
    // of the threat point touches the players kickable-area.
    TVector tToP = it->getPosition() - threat;
    double actionRadius = parameterView->SP_kickable_area;

    if(tToP.getLength() < actionRadius)
    {
      // Someone has the ball within his actionradius, the goal is covered.
      spans.clear();
      break;
    }

    double alfa = std::acos(actionRadius / tToP.getLength()) * 180.0 / M_PI;
    TVector delta1 = -(tToP.normalized());
    delta1 *= actionRadius;
    TVector delta2 = delta1;
    delta1.rotate(-alfa);
    delta2.rotate(alfa);

//      cout << "alfa: " << alfa;

    TLine line1 = TLine(threat, threat + tToP + delta1);
    TLine line2 = TLine(threat, threat + tToP + delta2);

    double s1, tStart;
    goalLine.getIntersection(line1, tStart, s1);
    double s2, tEnd;
    goalLine.getIntersection(line2, tEnd, s2);
    GoalieIntention::Span s;
    s.start = tStart;
    s.end = tEnd;

//      cout << ", tStart: " << tStart;
//      cout << ", tEnd: " << tEnd << ", s: " << s1 << ", " << s2 << endl;
    if(s1 > 0.0 && s2 > 0.0)
    {
      insertSpan(s, spans);
    }
  }

  // Initialize to defalutvalues.
  spanStart = TVector(-52.5, 0);
  spanEnd = TVector(-52.5, 0);

  list<GoalieIntention::Span>::iterator spanIt;
  double maxSpan = 0.0;
//    double t;
  for(spanIt = spans.begin(); spanIt != spans.end(); spanIt++)
  {
//      cout << "Cur span: " << spanIt->start << ", " << spanIt->end << endl;
    if( (spanIt->end - spanIt->start) > maxSpan )
    {
      maxSpan = spanIt->end - spanIt->start;
      spanStart = goalLine.getStart() + goalLine.getVector() * spanIt->start;
      spanEnd = goalLine.getStart() + goalLine.getVector() * spanIt->end;
    }
  }
}

// Uncomment either chooseSkill or generatePlan
/* Generates a plan. 
 * Returns an empty plan.
 *
 Plan GoalieIntention::generatePlan() {
 return Plan(); // Your plan
 }
*/

// Uncomment either chooseSkill or generatePlan
/** Chooses a skill. It works like a mini-decision maker.
 *
 */
Skill* GoalieIntention::chooseSkill()
{
  static InPenaltyArea inPenaltyArea;

  const BallState& ball = worldStateView->getWorldState().getBall();
  const AgentState& agent = worldStateView->getWorldState().getAgent();
  
  // Calculate the optimal cut-off-spot.

  // Calculation of actionradius.
  TVector bToA = agent.getPosition() - ball.getPosition();
  double timeToAgent = 
    floor(bToA.getLength() / parameterView->SP_ball_speed_max);
//    cout << "ETA: " << timeToAgent << endl;
  double actionRadius = 0.5 * parameterView->SP_catchable_area_l;
  if(timeToAgent > 5)
    actionRadius += (timeToAgent-5) 
      * 0.9 * parameterView->SP_player_speed_max;

  // The interesting parameters.
  // Use threat forwarding:
  // If the ball is passed to another player, use the predicted
  // interception point as the threat
//    const BallState& threat = statisticsView->getInterceptionBallState();
  const BallState& threat = ball;
  Display::instance()->addArrow(agent.getPosition(), threat.getPosition());
  TVector focusPoint = threat.getPosition();
  static TVector lastFocusPoint;
  focusPoint = (focusPoint + lastFocusPoint) / 2.0;
  lastFocusPoint = focusPoint;
  TVector spanStart;
  TVector spanEnd;
  
  calculateSpans(focusPoint, spanStart, spanEnd);
//    cout << "Start: " << spanStart
//         << ", End: " << spanEnd << endl;

  // Start and end vectors of the span to cover.
  TVector edgeVec1 = spanStart - focusPoint;
  TVector edgeVec2 = spanEnd - focusPoint;

  TVector minEdge;
  if(edgeVec1.getLength() < edgeVec2.getLength())
    minEdge = edgeVec1;
  else
    minEdge = edgeVec2;

  // Set the same length to both vectors.
  edgeVec2.setLength(edgeVec1.getLength());
  TVector edgeMid = (edgeVec1 + edgeVec2) / 2.0;

  // Calculate the base of the triangle.
  TVector baseVec = edgeVec2 - edgeVec1;

  // Calculate the sweet spot.
  TVector cutOffSpot = 
    2*actionRadius*edgeMid / baseVec.getLength();
  if(cutOffSpot.getLength() > minEdge.getLength())
    cutOffSpot.setLength(minEdge.getLength());
  cutOffSpot += focusPoint;

  double markDist = (TVector(-52.5, 0) - cutOffSpot).getLength();

  if(cutOffSpot.getX() > (-52.5 + 15))
    cutOffSpot.setX(-52.5 + 15);

//    Display* d = Display::instance();
//    d->addCircle(cutOffSpot, actionRadius);

  static FindBall findBall;
  static ClearBall clearBall;
  static FuzzyTeleport teleport;
  static CatchBall catchBall;
  static MarkPosition markPosition;
  static MoveToPosition moveToPosition;
  static Intercept intercept(10);

  if(gameView->isOurBall() 
     || inPenaltyArea() < FuzzyBool(0.75)
     || catchBall.isBanned())
    intercept.setGoalie(false);
  else
    intercept.setGoalie(true);

  if(ball.getPosition().getX() > 0)
  {
    moveToPosition.setPosition(TVector(-52, 0.0));
    if(moveToPosition.applicable())
      return &moveToPosition;
    return &findBall;
  }

  if(teleport.applicable()) {
    return &teleport;
  }

  if(catchBall.applicable() > FuzzyBool(0.3)) {
    return &catchBall;
  }

  if(clearBall.applicable()) {
    return &clearBall;
  }

  if(findBall.applicable(0.2)) {
    return &findBall;
  }

//    if(intercept.applicable())
//    {
//  //      cout << "Intercept!" << endl;
//      return &intercept;
//    }

//    moveToPosition.setPosition(cutOffSpot);

//    if( (cutOffSpot - agent.getPosition()).getLength() > 2.0)
//      moveToPosition.setSpeed(MoveToPosition::msVeryFast);
//    else
//      moveToPosition.setSpeed(MoveToPosition::msGainStamina);
//    return &moveToPosition;


//    const BallState& ball(worldStateView->getWorldState().getBall());
  const vector<PlayerState>& opps = 
    worldStateView->getWorldState().getOpponents();
  int minTime = 1000;
  for(unsigned int i = 0; i < opps.size(); i++)
  {
    int curTime = opps[i].getInterceptTime(ball);
    if(curTime != -1
       && curTime < minTime)
    {
      minTime = curTime;
    }
  }

  intercept.setMaxSteps(min(minTime+1, 10));
  if(intercept.applicable()) {
    return &intercept;
  }

  if(findBall.applicable(0.2)) {
    return &findBall;
  }

  // calculate goalie-position
  markPosition.setLookAtMark(false);
  TVector goal = TVector(-52.5, 0);
  float angle = abs((ball.getPosition() - goal).getAngle());
  float distance = 7.0 - 2.0*(angle/90.0);
  float xpos = -52.5 + 5.0*(angle/90.0);
  float ypos = 0;
  //     float ypos = ball.getPosition().getY() / parameterView->SP_pitch_width
  //       * parameterView->SP_goal_width;
  TVector markPos = TVector(xpos, ypos);
  TVector markDir = distance * (ball.getPosition() - markPos).normalized();
  Display::instance()->addArrow(markPos, markPos + markDir);
  markPosition.setDistance(markDist);
  markPosition.setPosition((spanStart + spanEnd) / 2.0);
  return &markPosition;
}

/** Models a player. 
 * This intention simply calls plr.doNull().
 * @param plr the player to be modelled
 */
void GoalieIntention::model(WorldState& ws)
{
  // Simple modelling.
  // Goalie is on a semi-cirle with a goal-width / 2 radius
  // in front of the goal.

  TVector gToB = ws.getBall().getPosition() - TVector(-52.5, 0);
  gToB = gToB.normalized();
  TVector desPos = TVector(-52.5, 0) + gToB * 5.0;
  TVector aToD = desPos - ws.getAgent().getPosition();

  AgentState agent = ws.getAgent();
  agent.setDirection(aToD.getAngle());
  agent.doDashDistance(aToD.getLength());
  ws.setAgent(agent);
}

Intention::Utility GoalieIntention::modelUtility(const WorldState& ws)
{
  if(ws.getAgent().getUniformNumber() == 1)
    return Intention::uVeryGood;
  return Intention::uSuck;
}

/** Checks if the utility of this intention.
 * uSuck - I'm not the goalie.
 * uVeryGood - I'm the goalie.
 */
Intention::Utility GoalieIntention::utility() {
  if(parameterView->CP_goalie)
    return Intention::uVeryGood;
  return Intention::uSuck;
}

/** Returns the persistence of this intention.
 * This intention is very persistent.
 * @return a float between zero (not persistent) and one (very persistent)
 */
float GoalieIntention::getPersistence() const {
  return 1.0; // Very persistent
}

/** Returns the ID that identifies this (type of) intention.
 */
IntentionId GoalieIntention::getID() const {
  return GOALIE_INTENTION_ID;
}

RS_END_NAMESPACE

/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "GoalieIntention.icc"
#endif







