/* -*- Mode: C++ -*-
 * $Id: MarkPlayer.h,v 1.6 2001/06/05 20:33:53 mikbr212 Exp $
 *
 * IDENTIFICATION
 *
 * Filename: MarkPlayer.h
 * Unitname: Framework/view
 * $Revision: 1.6 $
 * Created by: Fredrik Heintz 2000-09-12
 * Last modified by $Author: mikbr212 $ $Date: 2001/06/05 20:33:53 $
 *
 *
 * HISTORY
 *
 * $Log: MarkPlayer.h,v $
 * Revision 1.6  2001/06/05 20:33:53  mikbr212
 * Fixed doxygen-stubs for all classes
 *
 * Revision 1.5  2001/04/24 23:39:02  mikbr212
 * ?
 *
 * Revision 1.4  2001/04/24 19:12:28  mikbr212
 * added sticky
 *
 * Revision 1.3  2001/04/24 12:30:47  mikbr212
 * fixed
 *
 * Revision 1.2  2001/04/23 22:57:11  mikbr212
 * Killed and made a new skill. Not done yet
 *
 *
 */

/**
 * @file MarkPlayer.h
 * Include file for the view class MarkPlayer.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.6 $
 * $Author: mikbr212 $
 * $Date: 2001/06/05 20:33:53 $
 */

#ifndef MARKPLAYER_H
#define MARKPLAYER_H

#include "Skill.h"

#include "view_constants.h"

#include "WorldStateView.h"
#include "PassTreeView.h"
#include "CommunicationView.h"

#include "MarkPosition.h"

RS_BEGIN_NAMESPACE

/**
 * @class MarkPlayer MarkPlayer.h
 * This class implements the mark player skill
 *
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.6 $
 * $Author: mikbr212 $
 * $Date: 2001/06/05 20:33:53 $
 */

class MarkPlayer : public Skill {
public:
  MarkPlayer();
  virtual ~MarkPlayer();
  virtual Plan generatePlan();
  virtual FuzzyBool persistent();
  virtual FuzzyBool applicable();
  
  void setThreshold(const double min = 0.30);
  void setMaxPlayers(const unsigned int max = 6);

  void setBuffer(const double bufSize = 1.0);
  void setDistance(const double dist = 3.0);

  const PlayerState& getPlayer() const;

protected:
  /**
   * @class MarkRequest MarkPlayer.h
   * This class ...
   *
   * $Revision: 1.6 $
   * $Author: mikbr212 $
   * $Date: 2001/06/05 20:33:53 $
   */
  class MarkRequest {
  public:
    MarkRequest();
    ~MarkRequest();
    int requestCycle;
    int targetNo;
    int playerNo;
    bool isValid(const int currentCycle);
  };

  /**
   * @class IndexPredicate MarkPlayer.h
   * This class ...
   *
   * $Revision: 1.6 $
   * $Author: mikbr212 $
   * $Date: 2001/06/05 20:33:53 $
   */
  class IndexPredicate {
  public:
    IndexPredicate(const std::vector<float>& _values);

    bool operator()(const unsigned int left, const unsigned int right);
  private:
    const std::vector<float>& values;
  };
      

  bool isOpponentMarked(const PlayerState& op);
  bool isTeammateBusy(const PlayerState& tm);
  bool isPlayerMarkingPlayer(const PlayerState& plr,
			     const PlayerState& mark);
  bool findTarget();

  // attributes
  PlayerState target;

  unsigned int maxPlayers;
  double scoreThreshold;
  double distanceBuffer;
  double distance;
  std::vector<MarkRequest> requests;

  WorldStateView* worldStateView;
  PassTreeView* passTreeView;
  CommunicationView* comView;

  MarkPosition markSkill;

  /** Hide copy-constructor. */
  MarkPlayer(const MarkPlayer&);
  /** Hide assignment operator. */
  MarkPlayer& operator= (const MarkPlayer&);
};

RS_END_NAMESPACE

#endif // !MARKPLAYER_H

