#ifndef PASS_H
#define PASS_H


// Include the system header which definitions you use.
//#include <...>
//#include <...>

// Include the header files which definitions you use.
#include "Skill.h"

#include "KickBallToSpeedVector.h"
#include <list>

#include "ParameterView.h"
#include "PassTreeView.h"
#include "WorldStateView.h"
//  #include "TerritoryControlView.h"

#include "rs_log.h"

RS_BEGIN_NAMESPACE

//class ParameterView;
//class MarkerView;
//class AgentView;
//class BallView;

/**
 * @class Pass Pass.h
 * This class implements the pass skill
 *
 * <!--
 * @pre HaveBall, visible players.
 * @post Ball hopefully at the closest passable player.
 * @invariant ?!?!
 *
 * @todo Finetune the skill.
 * -->
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.7 $
 * $Author: andbo928 $
 * $Date: 2001/07/30 12:22:44 $
 */

class Pass : public Skill {
public:
  /** Default constructor. Here you can initialize the skill. */
  Pass();

  /** Destructor should clean up after the skill. */
  virtual ~Pass();
  
  /** This method is used to generate a plan for how the goal of
    the skill should be obtained by the agent. You can assume
    that the preconditions of the skill are true (this can be tested
    by calling the applicable() method, after you have implemented it). */
  virtual Plan generatePlan();

  /** This method should return a FuzzyBool telling the caller
    if the skill wants to be called again. */
  virtual FuzzyBool persistent();

  /** This method should implement the precondition specified in
   the description of the skill. */
  virtual FuzzyBool applicable();

  virtual void setVictim(const PlayerState& player);

  /** This method is used to first set the arguments of the skill
    and then generate a plan for how the goal of the skill should
    be obtained by the agent. It uses the methods setArguments and
    generatePlan(), so it should not be overriden unless it should
    made unavailable. */
protected:

  ParameterView* parameterView;
  PassTreeView* passTreeView;
  WorldStateView* worldStateView;
//    TerritoryControlView* territoryControlView;
  KickBallToSpeedVector kickBall;

  bool findPass();
  float passValue;
  int lastCallCycle;
  bool lastReturnValue;
  PlayerState victim;
  int calculatedTime;
  bool possible;
  
  Plan thePlan;

  /** Hide copy-constructor. */
  Pass(const Pass&);
  /** Hide assignment operator. */
  Pass& operator= (const Pass&);
};


RS_END_NAMESPACE

#endif // !PASS_H


