/* -*- Mode: C++ -*-
 * $Id: PassableIntention.h,v 1.10 2001/07/28 10:34:35 mikbr212 Exp $
 *
 * IDENTIFICATION
 *
 * $Revision: 1.10 $
 * Created by: Mikael Brnnstrm, Andreas Bckert, Andreas Sigfridsson
 * Last modified by $Author: mikbr212 $ $Date: 2001/07/28 10:34:35 $
 *
 *
 * HISTORY
 *
 * $Log: PassableIntention.h,v $
 * Revision 1.10  2001/07/28 10:34:35  mikbr212
 * Fixed looking
 *
 * Revision 1.9  2001/07/26 15:44:14  mikbr212
 * added offside avoidance and look at ball
 *
 * Revision 1.8  2001/07/26 12:19:33  mikbr212
 * Changed passtree
 *
 * Revision 1.7  2001/07/06 20:48:27  mikbr212
 * Added some reasonable utilities
 *
 * Revision 1.6  2001/07/06 14:47:40  andbo928
 * Patched to make it compile.
 *
 * Revision 1.5  2001/07/06 14:29:45  mikbr212
 * fixed utility
 *
 * Revision 1.4  2001/07/06 08:59:55  mikbr212
 * Made the some things work (but not yet in the new design)
 *
 * Revision 1.3  2001/07/03 15:24:19  andbo928
 * Added utility methods.
 *
 * Revision 1.2  2001/07/01 14:16:00  sigge
 * Added Mickes code (Intentions)
 *
 * Revision 1.1  2001/06/19 21:35:10  mikbr212
 * Created stubs
 *
 * Revision 1.1  2001/06/19 14:11:43  mikbr212
 * Created
 *
 */

/**
 * @file Intention.h
 * Definition file for the class Intention.
 *
 * @par Unit:
 *      Framework
 * $Revision: 1.10 $
 * $Author: mikbr212 $
 * $Date: 2001/07/28 10:34:35 $
 */

#ifndef PASSABLE_INTENTION_H
#define PASSABLE_INTENTION_H

#include "Intention.h"
#include "LookView.h"

RS_BEGIN_NAMESPACE

class PassTreeView;

/** 
 * @class PassableIntention PassableIntention.h
 * This intention is applicable for a player that wants to 
 * get passable. This is often the case when a player is
 * strategically placed but does not have the ball. 
 *
 * $Revision: 1.10 $
 * $Author: mikbr212 $
 * $Date: 2001/07/28 10:34:35 $
 */
class PassableIntention : public Intention {
public:
  PassableIntention();
  virtual ~PassableIntention();

  // Uncomment either chooseSkill or generatePlan
  virtual Plan generatePlan();

  virtual Intention::Utility utility();
  virtual Intention::Utility modelUtility(const WorldState& ws);

  virtual void model(WorldState&);

  virtual float getPersistence() const;

  virtual IntentionId getID() const;
  
protected:
  void initMovements();
  TVector getPosition();
  bool getLookAtBall();
  bool checkPreconditions();

  std::vector<TVector> movements;
  LookView* lookView;
  
  // Uncomment either chooseSkill or generatePlan
  //  virtual Skill* chooseSkill();

  // This method should not need to be overloaded
  //  virtual Plan getPlan(bool clear=false);

  // The following method is inherited:
  //  virtual Plan getPlan(bool clear=false);
  
  // The following views are inherited:
  //    ParameterView* parameterView;
  //    WorldStateView* worldStateView;
  //    PassTreeView* passTreeView;
  //    GameView* gameView;
  
private:
  /** Hide copy-constructor. */
  PassableIntention(const PassableIntention&);
  /** Hide assignment operator. */
  PassableIntention& operator= (const PassableIntention&);
};

RS_END_NAMESPACE

/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "PassableIntention.icc"
#endif

#endif // PASSABLE_INTENTION_H
