/* -*- Mode: C++ -*-
 * $Id: SkillStamina.h,v 1.1.1.1 2001/04/08 11:33:58 sigge Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: SkillStamina.h
 * Unitname: Framework/skill
 * $Revision: 1.1.1.1 $
 * Created by: Ola Lundqvist 2000-10-07
 * Last modified by $Author: sigge $ $Date: 2001/04/08 11:33:58 $
 *
 *
 * HISTORY
 *
 * $Log: SkillStamina.h,v $
 * Revision 1.1.1.1  2001/04/08 11:33:58  sigge
 * Imported sources
 *
 * Revision 1.4  2000/10/07 17:00:52  olalu526
 * Lot of changes.
 *
 * Revision 1.1  2000/10/07 14:28:27  olalu526
 * Added some skill files and fixed things.
 *
 *
 */

/**
 * @file SkillStamina.h
 * Include file for the skill class GotoBall.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.1.1.1 $
 * $Author: sigge $
 * $Date: 2001/04/08 11:33:58 $
 */


#ifndef SKILLSTAMINA_H
#define SKILLSTAMINA_H


#include "SkillWithArguments.h"


RS_BEGIN_NAMESPACE


/**
 * @class SkillStamina SkillStamina.h
 * The first sentence is a brief description of the skill class SkillStamina.
 * The rest is a longer description of the skill class SkillStamina.
 * Use it to describe the skill!
 *
 * @todo Things that should be done...
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.1.1.1 $
 * $Author: sigge $
 * $Date: 2001/04/08 11:33:58 $
 */
class SkillStamina : public SkillWithArguments {
public:
  /** Default constructor. Here you can initialize the skill. */
  SkillStamina();

  /** Destructor should clean up after the skill. */
  virtual ~SkillStamina();
  
  /** This method is used to generate a plan for how the goal of
    the skill should be obtained by the agent. You can assume
    that the preconditions of the skill are true (this can be tested
    by calling the applicable() method, after you have implemented it). */
  virtual Plan generatePlan() = 0;

  /** This method should return a FuzzyBool telling the caller
    if the skill wants to be called again. */
  virtual FuzzyBool persistent() = 0;

  /** This method should implement the precondition specified in
   the description of the skill. */
  virtual FuzzyBool applicable() = 0;

  /** Sets the maximum speed the agent can run. */
  virtual void setMaxSpeed(Float);

  /** Gets the maximum speed the agent can run. */
  virtual Float getMaxSpeed();

  /** Sets emergency mode. */
  virtual void setEmergency(FuzzyBool);

  /** Gets whether or not it is emergency mode. */
  virtual FuzzyBool getEmergency();

  /** Sets the maximum stamina used per cycle. */
  virtual void setMaxUseStaminaPerCycle(Float);

  /** Gets the maximum stamina used per cycle. */
  virtual Float getMaxUseStaminaPerCycle() const;

  /** Sets the maximum stamina used per cycle in emergency mode. */
  virtual void setEmergencyMaxUseStaminaPerCycle(Float);

  /** Gets the maximum stamina used per cycle in emergency mode. */
  virtual Float getEmergencyMaxUseStaminaPerCycle() const;

  /** Sets the maximum amount of stamina to use. */
  virtual void setMaxUseStamina(Float);

  /** Gets the maximum amount of stamina to use. */
  virtual Float getMaxUseStamina() const;

  /** Sets the minumum level of stamina in non emergency mode. */
  virtual void setMinStamina(Float);

  /** Gets the minumum level of stamina in non emergency mode. */
  virtual Float getMinStamina() const;

  /** Sets the minumum level of stamina in emergency mode. */
  virtual void setEmergencyMinStamina(Float);

  /** Gets the minumum level of stamina in emergency mode. */
  virtual Float getEmergencyMinStamina() const;

  /** Get the default min stamina level. */
  Float lowerEffortLimit() const;

  /** Get the default emergency min stamina level. */
  Float looseRecoverLimit() const;

private:
  FuzzyBool emergency;
  Float maxSpeed;
  Float emergencyMaxUseStaminaPerCycle;
  Float maxUseStaminaPerCycle;
  Float emergencyMinStamina;
  Float minStamina;
  ParameterView *parameterView;
  AgentView *agentView;

  /** Hide copy-constructor. */
  SkillStamina(const SkillStamina&);
  /** Hide assignment operator. */
  SkillStamina& operator= (const SkillStamina&);
};


RS_END_NAMESPACE


// Remove from here 
/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "SkillStamina.icc"
#endif
// To here if you do not have any inline definitions!

#endif // !SKILLSTAMINA_H
