#ifndef SUPERKICK_H
#define SUPERKICK_H


// Include the system header which definitions you use.
//#include <...>
//#include <...>

// Include the header files which definitions you use.
#include "Skill.h"
#include "view_constants.h"
#include "rs_log.h"
#include "WorldStateView.h"

RS_BEGIN_NAMESPACE

//class ParameterView;
//class MarkerView;
//class AgentView;
//class BallView;

/**
 * @class SuperKick SuperKick.h
 * This class implements the pass skill
 *
 * <!--
 * @pre HaveBall, visible players.
 * @post Ball hopefully at the closest passable player.
 * @invariant ?!?!
 *
 * @todo Finetune the skill.
 * -->
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.3 $
 * $Author: andbo928 $
 * $Date: 2001/04/15 23:25:45 $
 */

class SuperKick : public Skill {
public:
  /** Default constructor. Here you can initialize the skill. */
  SuperKick(float dir = 0);

  /** Destructor should clean up after the skill. */
  virtual ~SuperKick();
  
  /** This method is used to generate a plan for how the goal of
    the skill should be obtained by the agent. You can assume
    that the preconditions of the skill are true (this can be tested
    by calling the applicable() method, after you have implemented it). */
  virtual Plan generatePlan();

  /** This method should return a FuzzyBool telling the caller
    if the skill wants to be called again. */
  virtual FuzzyBool persistent();

  /** This method should implement the precondition specified in
   the description of the skill. */
  virtual FuzzyBool applicable();

  void setDirection(float dir);
  float getDirection();

protected:
  WorldStateView* worldView;

  bool opKickSimple(WorldState& ws, Step& opStep);
  bool opKickPos1(WorldState& ws, Step& opStep);
  bool opKickPos2(WorldState& ws, Step& opStep);
  bool opKickPos3(WorldState& ws, Step& opStep);
  bool opKickPos4(WorldState& ws, Step& opStep);

  double destDir;
  bool goalFound(const WorldState& ws);
  TVector getPos(int n);

  bool genPlanIter(const WorldState& curWs, 
		   int recDepth,
		   Step& theStep);

  /** Hide copy-constructor. */
  SuperKick(const SuperKick&);
  /** Hide assignment operator. */
  SuperKick& operator= (const SuperKick&);
};


RS_END_NAMESPACE

#endif // !SUPERSCORE_H

