#ifndef SUPERSCORE_H
#define SUPERSCORE_H


// Include the system header which definitions you use.
//#include <...>
//#include <...>

// Include the header files which definitions you use.
#include "Skill.h"

#include "CoordVector.h"
#include "CoordPoint.h"
#include "view_constants.h"
#include "ParameterView.h"
#include "MarkerView.h"
#include "AgentView.h"
#include "BallView.h"
#include "TerritoryControlView.h"
#include "FuzzyPlayerView.h"
#include "FuzzyBallView.h"
#include "rs_log.h"
#include "KickBallToSpeedVectorMulti.h"
#include "MoveToPositionFacingBall.h"

RS_BEGIN_NAMESPACE

//class ParameterView;
//class MarkerView;
//class AgentView;
//class BallView;

/**
 * @class SuperScore SuperScore.h
 * This class implements the pass skill
 *
 * <!--
 * @pre HaveBall, visible players.
 * @post Ball hopefully at the closest passable player.
 * @invariant ?!?!
 *
 * @todo Finetune the skill.
 * -->
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.1.1.1 $
 * $Author: sigge $
 * $Date: 2001/04/08 11:33:58 $
 */

class SuperScore : public Skill {
public:
  /** Default constructor. Here you can initialize the skill. */
  SuperScore();

  /** Destructor should clean up after the skill. */
  virtual ~SuperScore();
  
  /** This method is used to generate a plan for how the goal of
    the skill should be obtained by the agent. You can assume
    that the preconditions of the skill are true (this can be tested
    by calling the applicable() method, after you have implemented it). */
  virtual Plan generatePlan();

  /** This method should return a FuzzyBool telling the caller
    if the skill wants to be called again. */
  virtual FuzzyBool persistent();

  /** This method should implement the precondition specified in
   the description of the skill. */
  virtual FuzzyBool applicable();

protected:

  ParameterView* parameterView;
  MarkerView* markerView;
  AgentView* agentView;
  FuzzyBallView* ballView;
  FuzzyPlayerView* playerView;

  Vector calcKickAcc(Vector kick, Point bpos, Point agpos, float agdir); 
  Vector calcKickVector(Vector acc, Point bpos, Point agpos, float agdir);
 
  Plan genPlan(Point bpos, Vector bspeed, Point agpos, Vector agspeed, float agdir, Plan curPlan);

  /** Hide copy-constructor. */
  SuperScore(const SuperScore&);
  /** Hide assignment operator. */
  SuperScore& operator= (const SuperScore&);
};


RS_END_NAMESPACE

#endif // !SUPERSCORE_H

