/* -*- Mode: C++ -*-
 * $Id: TakeBallIntention.h,v 1.9 2001/07/30 17:20:35 andbo928 Exp $
 *
 * IDENTIFICATION
 *
 * $Revision: 1.9 $
 * Created by: Mikael Brnnstrm, Andreas Bckert, Andreas Sigfridsson
 * Last modified by $Author: andbo928 $ $Date: 2001/07/30 17:20:35 $
 *
 *
 * HISTORY
 *
 * $Log: TakeBallIntention.h,v $
 * Revision 1.9  2001/07/30 17:20:35  andbo928
 * Now ignores goalie when calculating teammates. Despammed.
 *
 * Revision 1.8  2001/07/06 14:30:29  sigge
 * Fixed abstraction bug.
 *
 * Revision 1.7  2001/07/06 14:07:40  andbo928
 * Removed worldstate as utility parameter.
 *
 * Revision 1.6  2001/07/06 14:03:09  andbo928
 * Added modelutility
 *
 * Revision 1.5  2001/07/06 08:59:55  mikbr212
 * Made the some things work (but not yet in the new design)
 *
 * Revision 1.4  2001/07/05 12:23:31  andbo928
 * More agressive play.
 *
 * Revision 1.3  2001/07/04 16:44:48  andbo928
 * Works somewhat better now, not perfect yet.
 *
 * Revision 1.2  2001/07/03 15:24:19  andbo928
 * Added utility methods.
 *
 * Revision 1.1  2001/06/19 14:11:43  mikbr212
 * Created
 *
 */

/**
 * @file Intention.h
 * Definition file for the class Intention.
 *
 * @par Unit:
 *      Framework
 * $Revision: 1.9 $
 * $Author: andbo928 $
 * $Date: 2001/07/30 17:20:35 $
 */

#ifndef TAKE_BALL_INTENTION_H
#define TAKE_BALL_INTENTION_H

#include "Intention.h"

RS_BEGIN_NAMESPACE

/** 
 * @class TakeBallIntention TakeBallIntention.h
 * This class is take ball intention. It is applicable
 * for a ball that is not moving that much, that is
 * when it is just to take the ball (ie not receiving a pass etc).
 *
 * $Revision: 1.9 $
 * $Author: andbo928 $
 * $Date: 2001/07/30 17:20:35 $
 */
class TakeBallIntention : public Intention {
public:
  TakeBallIntention();
  virtual ~TakeBallIntention();

  virtual void model(WorldState&);

  // Uncomment if the agent should be threated specially
  //virtual Utility utility();
  virtual Utility utility();

  /** Return @c the utility value of the intention.
   * This should be a light weight method, since this
   * will be called for all other players.
   */
  virtual Utility modelUtility(const WorldState&);

  virtual float getPersistence() const;

  virtual IntentionId getID() const;
  
protected:
  virtual Skill* chooseSkill();

  float getSpeed(const float myDist, const float opDist);
  TVector getCutOffGoalAnglePosition(Team team);

  double underestimate(double d);
  double overestimate(double d);
  void findBetter(int plrTime, 
		  bool ignoreGoalie,
		  const BallState& ball, 
		  const vector<PlayerState>& plrs, 
		  int& better, int& similar);

  // The following views are inherited:
  //    ParameterView* parameterView;
  //    WorldStateView* worldStateView;
  //    PassTreeView* passTreeView;
  //    GameView* gameView;
  
private:
  /** Hide copy-constructor. */
  TakeBallIntention(const TakeBallIntention&);
  /** Hide assignment operator. */
  TakeBallIntention& operator= (const TakeBallIntention&);
};

RS_END_NAMESPACE

#endif // TAKE_BALL_INTENTION_H
