/* -*- Mode: C++ -*-
 * $Id: GameView.cc,v 1.6 2001/07/06 10:00:22 mikbr212 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: GameView.cc
 * Unitname: Framework
 * $Revision: 1.6 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: mikbr212 $ $Date: 2001/07/06 10:00:22 $
 *
 *
 * HISTORY
 *
 * $Log: GameView.cc,v $
 * Revision 1.6  2001/07/06 10:00:22  mikbr212
 * Merged updates and resolved conflicts (a lot)
 *
 * Revision 1.5  2001/06/19 15:23:23  mikbr212
 * Fixed bug in isOurBall and isTheirBall
 *
 * Revision 1.4  2001/06/19 14:15:35  mikbr212
 * Added isOurBall and isTheirBall.
 *
 * Revision 1.3  2001/04/20 16:21:58  sigge
 * Replaced getPlayerMode with getPlayerModes.
 *
 * Revision 1.2  2001/04/20 14:45:05  sigge
 * Modified GameView to have a vector of PlayModes. Is not currently used, yet.
 *
 * Revision 1.1.1.1  2001/04/08 11:33:58  sigge
 * Imported sources
 *
 * Revision 1.1  2000/09/21 13:46:46  frehe
 * Move the RoboSoc views from rsviews to the rsframework library.
 *
 * Revision 1.2  2000/09/13 00:08:08  frehe
 * Updated the views to inherit from the ViewWithHistory instead and also changed those views that used a static pointer to itself to do process the data in the sensor interface to use the new forEach methods implemented in the FrameworkSensorInterface.
 *
 * Revision 1.1  2000/09/04 15:27:23  frehe
 * Imported the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file GameView.cc
 * Implementation file for the class GameView.
 *
 * @par Unit:
 *      Framework
 * $Revision: 1.6 $
 * $Author: mikbr212 $
 * $Date: 2001/07/06 10:00:22 $
 */


#include "GameView.h"
#include <algorithm>

RS_BEGIN_NAMESPACE


GameView::GameView(const ViewId id,
		   UInt16 hist_len)
  : ViewWithHistory(id, hist_len),
    playModes(hist_len, std::vector<PlayMode>(1, PM_UNKNOWN))
{
}


GameView::~GameView()
{
}
   

void GameView::myNewCycle()
{
  // Update the current observation.
  increaseIndex();
  
  // Copy the play mode
  playModes[getIndex(NOW)] = std::vector<PlayMode>(1, playModes[getIndex(PREV)].back());
}


void GameView::myUpdateAfterHear()
{
  // Save the current play mode.
  playModes[getIndex(NOW)].push_back(sensorInterface->getPlayModes());
}


void GameView::myUpdateAfterInit()
{
  // Set the play mode to the initial play mode.
  playModes[getIndex(NOW)].push_back(sensorInterface->getPlayModes());
}

bool GameView::isOurBall(const Int hist) const {
  const std::vector<PlayMode>& pms(getPlayModes(hist));

  if(find(pms.begin(), pms.end(), PM_OUR_OFFSIDE_KICK) != pms.end() ||
     find(pms.begin(), pms.end(), PM_OUR_KICK_OFF) != pms.end() ||
     find(pms.begin(), pms.end(), PM_OUR_KICK_IN) != pms.end() ||
     find(pms.begin(), pms.end(), PM_OUR_FREE_KICK) != pms.end() ||
     find(pms.begin(), pms.end(), PM_OUR_CORNER_KICK) != pms.end() ||
     find(pms.begin(), pms.end(), PM_OUR_GOAL_KICK) != pms.end() ||
     find(pms.begin(), pms.end(), PM_OUR_GOALIE_GOT_BALL) != pms.end())
    return true;
  return false;
}

bool GameView::isTheirBall(const Int hist) const {
  const std::vector<PlayMode>& pms(getPlayModes(hist));

  if(find(pms.begin(), pms.end(), PM_THEIR_OFFSIDE_KICK) != pms.end() ||
     find(pms.begin(), pms.end(), PM_THEIR_KICK_OFF) != pms.end() ||
     find(pms.begin(), pms.end(), PM_THEIR_KICK_IN) != pms.end() ||
     find(pms.begin(), pms.end(), PM_THEIR_FREE_KICK) != pms.end() ||
     find(pms.begin(), pms.end(), PM_THEIR_CORNER_KICK) != pms.end() ||
     find(pms.begin(), pms.end(), PM_THEIR_GOAL_KICK) != pms.end() ||
     find(pms.begin(), pms.end(), PM_THEIR_GOALIE_GOT_BALL) != pms.end())
    return true;
  return false;
}

RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "GameView.icc"
#endif
