/* -*- Mode: C++ -*-
 * $Id: Display.h,v 1.8 2001/07/11 16:35:50 sigge Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: Display.h
 * Unitname: Framework/example/FCFoo
 * $Revision: 1.8 $
 * Created by: Johan Thapper 1999-xx-xx
 * Last modified by $Author: sigge $ $Date: 2001/07/11 16:35:50 $
 *
 *
 * HISTORY
 *
 * $Log: Display.h,v $
 * Revision 1.8  2001/07/11 16:35:50  sigge
 * Modified Monitor protocol for size and decoding speed.
 *
 * Revision 1.7  2001/07/04 13:21:36  sigge
 * Added role display.
 *
 * Revision 1.6  2001/04/23 10:22:22  andbo928
 * Cleaned up.
 *
 * Revision 1.5  2001/04/20 14:46:38  sigge
 * Now uses the username in the fifo names, to avoid conflicts on shared machines (astmatix). Also added addArrow with TLine
 *
 * Revision 1.4  2001/04/19 12:33:32  mikbr212
 * fixed
 *
 * Revision 1.3  2001/04/19 12:27:12  mikbr212
 * Changed arrow-time to cycles (int)
 *
 * Revision 1.2  2001/04/13 11:18:28  mikbr212
 * Added ::instance()
 *
 * Revision 1.1  2001/04/13 10:41:29  mikbr212
 * Moved Display
 *
 * Revision 1.3  2001/04/11 18:54:52  andbo928
 * Fixed some stuff.
 *
 * Revision 1.2  2001/04/11 16:24:35  andbo928
 * Fixed display to show a lot of thingys...
 *
 * Revision 1.2  2001/04/11 16:07:48  andbo928
 * Fixed display to show a lot of thingys...
 *
 * Revision 1.1.1.1  2001/04/08 11:33:58  sigge
 * Imported sources
 *
 * Revision 1.3  2000/10/24 15:15:45  frehe
 * Made minor updates and tests.
 *
 * Revision 1.2  2000/10/11 18:40:29  frehe
 * Upgraded FCFoo.
 *
 * Revision 1.1  2000/09/05 21:12:54  frehe
 * Made FCFoo compile with RoboSoc 2.5.0
 *
 */

/**
 * @file Display.h
 * Include file for the class Display.
 *
 * @par Unit:
 *      Framework/example/FCFoo
 * $Revision: 1.8 $
 * $Author: sigge $
 * $Date: 2001/07/11 16:35:50 $
 */

#ifndef DISPLAY_H
#define DISPLAY_H

#include "TVector.h"

#include <string>
#include <fstream>

#include "ViewManager.h"
#include "WorldStateView.h"
#include "ParameterView.h"
#include "GameView.h"
#include "MarkerView.h"
//  #include "AgentView.h"
//  #include "BallView.h"
//  #include "PlayerView.h"
//  #ifdef RS_USE_COACH_DEBUGGING
//  #  include "CoachView.h"
//  #endif
#include "TLine.h"


//  #define CLEAR_SCREEN cout << char(27) << "[2J"
//  #define HOME_CURSOR cout << char(27) << "[H"
//  #define MOVE_CURSOR(x,y) cout << char(27) << '[' << x << ',' << y << 'H'


RS_BEGIN_NAMESPACE


/**
 * @class Display Display.h
 * Example class for what a debug information display could look like.
 *
 * <!-- @todo-->
 *
 * @par Unit:
 *      Library
 * $Revision: 1.8 $
 * $Author: sigge $
 * $Date: 2001/07/11 16:35:50 $
 */
class Display {
public:
  enum Color
  {
    cBlack = 0,
    cBlue = 1,
    cCyan = 2,
    cDarkgray = 3,
    cGray = 4,
    cLightgray = 5,
    cMagenta = 6,
    cOrange = 7,
    cPink = 8,
    cRed = 9,
    cWhite = 10,
    cYellow = 11
  };

  static Display* instance();

  void setLastSkill(const std::string&);
  void draw();
  void addArrow(const TVector& start, 
		const TVector& end, 
		int duration = 1,
		Display::Color color = cBlack);
  void addArrow(const TLine& line,
		int duration = 1,
		Display::Color color = cBlack);
  void addCircle(const TVector center,
		 float radius,
		 int duration = 1,
		 Display::Color color = cBlack);
  void addMessage(string msg);
  void setEnabled(bool newStatus);


protected:
  Display();
  ~Display();
  static Display* theOneAndOnly;
  
  ViewManager* viewManager;
#ifdef RS_USE_COACH_DEBUGGING
  CoachView* coachView;
#endif
  WorldStateView* worldView;
  
  // Arrows
  vector<TVector> arrowStart;
  vector<TVector> arrowEnd;
  vector<int> arrowDuration;
  vector<Display::Color> arrowColor;

  // Circles
  vector<TVector> circleCenters;
  vector<float> circleRadii;
  vector<int> circleDuration;
  vector<Display::Color> circleColor;

  // Messages
  vector<string> messages;

  // Last skill
  std::string lastSkill;

  bool enabled;
  int fd;
  int is_open;

private:
  void PutFloat(float f);
  void PutFloatByte(float f);
  void PutShort(int i);
  void PutByte(int i);
  void PutPlayer(const PlayerState& player, int team, int me);
};


RS_END_NAMESPACE


#endif // !DISPLAY_H

