/* -*- Mode: C++ -*-
 * $Id: FuzzyBallView.cc,v 1.3 2001/04/20 16:21:58 sigge Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: FuzzyBallView.cc
 * Unitname: Framework/view
 * $Revision: 1.3 $
 * Created by: Mikael Brnnstrm, Andreas Sigfridsson, Andreas Bckert
 * Last modified by $Author: sigge $ $Date: 2001/04/20 16:21:58 $
 *
 *
 * HISTORY
 *
 * $Log: FuzzyBallView.cc,v $
 * Revision 1.3  2001/04/20 16:21:58  sigge
 * Replaced getPlayerMode with getPlayerModes.
 *
 * Revision 1.2  2001/04/11 12:11:01  andbo928
 * *** empty log message ***
 *
 * Revision 1.1.1.1  2001/04/08 11:33:59  sigge
 * Imported sources
 *
 * Revision 1.1  2000/11/20 12:41:27  mikbr212
 * File was corrupt in the cvs repository. Removed and added file again.
 *
 * Revision 1.8  2000/11/19 17:14:52  mikbr212
 * added kick-stuff
 *
 * Revision 1.7  2000/11/19 16:50:32  mikbr212
 * fixed X and Y stuff in afterSee
 *
 * Revision 1.6  2000/11/19 16:46:49  mikbr212
 * now uses absolute vectors
 *
 * Revision 1.5  2000/11/19 15:32:36  mikbr212
 * fixed more stuff
 *
 * Revision 1.4  2000/11/19 15:23:50  mikbr212
 * added more stuff
 *
 * Revision 1.3  2000/11/19 12:51:00  mikbr212
 * added stuff
 *
 * Revision 1.2  2000/11/19 11:58:14  mikbr212
 * fixed stuff
 *
 * Revision 1.1  2000/11/19 11:38:49  mikbr212
 * created
 *
 */

/**
 * @file FuzzyBallView.cc
 * Implementation file for the class FuzzyBallView.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.3 $
 * $Author: sigge $
 * $Date: 2001/04/20 16:21:58 $
 */


#include "FuzzyBallView.h"

RS_BEGIN_NAMESPACE

FuzzyBallView::FuzzyBallView(const ViewId id,
		   const UInt hist_len)
  : ViewWithHistory(id, hist_len),
    parameterView(NULL),
    commandView(NULL),
    agentView(NULL),
    absSpeedHist(hist_len, VectorUCO()),
    absPosHist(hist_len, VectorUCO()),
    relSpeedHist(hist_len, VectorUCO()),
    relPosHist(hist_len, VectorUCO()),
    obsRelSpeedHist(hist_len, VectorUCO()),
    obsRelPosHist(hist_len, VectorUCO()),
    obsDistChangeHist(hist_len, FloatUnknown()),
    obsDirChangeHist(hist_len, AngleDegUnknown())
{
  getView(parameterView, PARAMETER_VIEW_ID);
  getView(gameView, GAME_VIEW_ID, hist_len);
  getView(commandView, COMMAND_VIEW_ID, hist_len);
  getView(agentView, AGENT_VIEW_ID, parameterView, hist_len);
}

FuzzyBallView::~FuzzyBallView()
{
  viewManager->releaseView(PARAMETER_VIEW_ID);
  viewManager->releaseView(GAME_VIEW_ID);
  viewManager->releaseView(COMMAND_VIEW_ID);
  viewManager->releaseView(AGENT_VIEW_ID);
}

void FuzzyBallView::myUpdateAfterInit()
{
  // initially the ball is in the middle of the field
  absSpeedHist[getIndex(NOW)] = VectorUCO(0,0);
  absPosHist[getIndex(NOW)] = VectorUCO(0,0);
  fixUnknownToNoConfidence(relPosHist[getIndex(NOW)]);
  fixUnknownToNoConfidence(relSpeedHist[getIndex(NOW)]);
}


// NewCycle is called *before* the values in CSensorInterface
// are erased, so this is the last chance to use them...
void FuzzyBallView::myNewCycle() {
  commandView->newCycle(currentUpdateNumber);
  agentView->newCycle(currentUpdateNumber);

  // move ball and update speed
  absSpeedHist[getIndex(NOW)] += commandView->getBallAcceleration(PREV);

  if(absSpeedHist[getIndex(NOW)].getLength().getConfidence() > 0.5)
    absPosHist[getIndex(NEXT)] = absPosHist[getIndex(NOW)] + absSpeedHist[getIndex(NOW)];
  else
    absPosHist[getIndex(NEXT)] = absPosHist[getIndex(NOW)];

  absSpeedHist[getIndex(NEXT)] = absSpeedHist[getIndex(NOW)] * parameterView->SP_ball_decay;

  // Compute relative position and speed from absolute 
  relPosHist[getIndex(NEXT)] = absPosHist[getIndex(NEXT)] -
    VectorUCO(agentView->getAgentPosition(NOW));
  relPosHist[getIndex(NEXT)].rotate(-agentView->getBodyDirection(NOW));

  relSpeedHist[getIndex(NEXT)] = absSpeedHist[getIndex(NEXT)] -
    agentView->getSpeedVector(NOW);
  relSpeedHist[getIndex(NEXT)].rotate(-agentView->getBodyDirection(NOW));

  // Set not yet observed values to unknown
  obsRelPosHist[getIndex(NEXT)] = VectorUCO();
  obsRelSpeedHist[getIndex(NEXT)] = VectorUCO();
  obsDistChangeHist[getIndex(NEXT)] = FloatUnknown();
  obsDirChangeHist[getIndex(NEXT)] = AngleDegUnknown();

  // Fix unknown (unknown = no confidence)
  fixUnknownToNoConfidence(absPosHist[getIndex(NEXT)]);
  fixUnknownToNoConfidence(absSpeedHist[getIndex(NEXT)]);
  fixUnknownToNoConfidence(relPosHist[getIndex(NEXT)]);
  fixUnknownToNoConfidence(relSpeedHist[getIndex(NEXT)]);

  // degrade confidence
  degradeVector(absSpeedHist[getIndex(NEXT)],0.9);
  degradeVector(relSpeedHist[getIndex(NEXT)],0.9);

  degradeVector(absPosHist[getIndex(NEXT)],0.9);
  degradeVector(relPosHist[getIndex(NEXT)],0.9);

//    cout << "FBV - newCycle:";
//    cout << " ballPos conf: " << absPosHist[getIndex(NOW)].getLength().getConfidence() << endl;

  increaseIndex();
}


void FuzzyBallView::myUpdateAfterSee()
{
  agentView->updateAfterSee(currentUpdateNumber);
  
  if(sensorInterface->sawBall()) {
    FloatUnknown dist = sensorInterface->getBallDistance();
    AngleDegUnknown dir = sensorInterface->getBallDirection();
    FloatUnknown distChange = sensorInterface->getBallDistanceChange();
    AngleDegUnknown dirChange = AngleDegUnknown(sensorInterface->getBallDirectionChange());

//      cout << "FBV - afterSee:";
//      cout << " ballPos observerd " << endl;

    obsDistChangeHist[getIndex(NOW)] = distChange;
    obsDirChangeHist[getIndex(NOW)] = dirChange;
    obsRelPosHist[getIndex(NOW)] = VectorUCO(dir,dist);
    obsRelPosHist[getIndex(NOW)].rotate(agentView->getNeckDirection(NOW)); 

    // Convert to absolute position
    absPosHist[getIndex(NOW)] = obsRelPosHist[getIndex(NOW)];
    absPosHist[getIndex(NOW)].rotate(agentView->getBodyDirection(NOW));
    absPosHist[getIndex(NOW)] += agentView->getAgentPosition(NOW);

    /*
     * Absolute Velocity = Player Velocity + Relative Velocity
     *
     * Relative Velocity.x = DistChg * Unit.x - ( DirChg * pi/180 ) * Dist * Unit.y
     * Relative Velocity.y = DistChg * Unit.y + ( DirChg * pi/180 ) * Dist * Unit.x
     *
     * Unit = Relative Position / Dist
     *
     * Relative Position = Absolute Position - Player Position
     */

//      if(obsRelPosHist[getIndex(PREV)].isKnown() && 
//         obsRelPosHist[getIndex(NOW)].isKnown() &&
//         obsRelPosHist[getIndex(NOW)].getVector().getLength() < parameterView->SP_visible_distance) {
//        absSpeedHist[getIndex(NOW)] = absPosHist[getIndex(NOW)] - absPosHist[getIndex(PREV)];
//        absSpeedHist[getIndex(NOW)] *= parameterView->SP_ball_decay;

//        // convert to relative speed
//        relSpeedHist[getIndex(NOW)] = absSpeedHist[getIndex(NOW)];
//        relSpeedHist[getIndex(NOW)] -= agentView->getSpeedVector(NOW);
//        relSpeedHist[getIndex(NOW)].rotate(-agentView->getBodyDirection(NOW)); 

//      } else 
      
    if(distChange.isKnown() && dirChange.isKnown() && dir.isKnown()) {
      Point unit = Point(Vector(dir.getAngle(), 1)); 
      Float tmp = dirChange.getAngle().getRad() * dist.getValue();
      Point speed(distChange.getValue() * unit.getX() - tmp * unit.getY(),
		  distChange.getValue() * unit.getY() + tmp * unit.getX());

      //      cout << " ballSpeed obs";
      
      // neck dir should always be known
      relSpeedHist[getIndex(NOW)] = VectorUCO(speed);
      relSpeedHist[getIndex(NOW)].rotate(agentView->getNeckDirection(NOW)); 
      obsRelSpeedHist[getIndex(NOW)] = relSpeedHist[getIndex(NOW)];

      // Convert to absolute speed
      absSpeedHist[getIndex(NOW)] = obsRelSpeedHist[getIndex(NOW)];
      absSpeedHist[getIndex(NOW)].rotate(agentView->getBodyDirection(NOW));
      absSpeedHist[getIndex(NOW)] += agentView->getSpeedVector(NOW);

//        cout << "avGsp:" << (agentView->getSpeedVector(NOW).isKnown() ? "known" : "unkown");
//        cout << "avBdr:" << (agentView->getSpeedVector(NOW).isKnown() ? "known" : "unkown") << endl;

//        cout << "Fb Abs:" << absSpeedHist[getIndex(NOW)] << endl;
//        cout << "Fb Rel:" << relSpeedHist[getIndex(NOW)] << endl;
    } else {
      // use estimation from last cycle
    }
  } else {
    // Remove ball if I should have seen it
    Float delta = 45;
    if(agentView->getViewWidth(NOW) == VW_WIDE)
      delta = 90;
    else if(agentView->getViewWidth(NOW) == VW_NARROW)
      delta = 22.5;
    
    delta *= 0.95; // Allow 5% noise at the edges

    if(abs(relPosHist[getIndex(NOW)].getAngle()) < delta) {
//       degradeVector(absSpeedHist[getIndex(NOW)],0);
//       degradeVector(relSpeedHist[getIndex(NOW)],0);
      
//       degradeVector(absPosHist[getIndex(NOW)],0);
//       degradeVector(relPosHist[getIndex(NOW)],0);
    }
    
  }
  
  //  cout << endl;

  // Fix unknown (unknown = no confidence)
  fixUnknownToNoConfidence(absPosHist[getIndex(NOW)]);
  fixUnknownToNoConfidence(absSpeedHist[getIndex(NOW)]);
  fixUnknownToNoConfidence(relPosHist[getIndex(NOW)]);
  fixUnknownToNoConfidence(relSpeedHist[getIndex(NOW)]);
}


void FuzzyBallView::myUpdateAfterHear()
{
//    agentView->updateAfterHear(currentUpdateNumber);
//    //markerView->updateAfterHear(currentUpdateNumber);

//    PlayMode pm = gameView->getPlayModes(NOW);
//    if ( pm == PM_BEFORE_KICK_OFF ||
//         pm == PM_KICK_OFF ||
//         pm == PM_OUR_KICK_OFF ||
//         pm == PM_THEIR_KICK_OFF ) {
//      theBall[getIndex(NOW)].setAbsoluteSpeedVector(VectorUCO(0,0));
//      //theBall[getIndex(NOW)].setPosition(Point(0,0));
//      theBall[getIndex(NOW)].setVector(VectorUCO(agentView->getAgentPosition(NOW), PointUCO(0,0)).
//  				     rotate(-agentView->getFaceDirection(NOW)));

//    }
}

void FuzzyBallView::degradeVector(VectorUCO& vec, Float f) {
  FloatUCO l;
  AngleDegUCO a;

  a = vec.getAngle();
  a.setConfidence(a.getConfidence() * f);
  vec.setAngle(a);

  l = vec.getLength();
  l.setConfidence(l.getConfidence() * f);
  vec.setLength(l);
}

void FuzzyBallView::fixUnknownToNoConfidence(VectorUCO& vec) { 
  if(!vec.getAngle().isKnown()) {
    AngleDegUCO ang(0);
    ang.setConfidence(0);
    vec.setAngle(ang);
  }
  if(!vec.getLength().isKnown()) {
    FloatUCO len = 0;
    len.setConfidence(0);
    vec.setLength(len);
  }
}

void FuzzyBallView::myUpdateAfterCommand()
{
  /* Since we currently send all commands last in the cycle, even
     immediate actions, we can do all the updating at the begining
     of the new cycle. */
}

void FuzzyBallView::estimate()
{
  /**
   * Absolute Velocity = Player Velocity + Relative Velocity
   *
   * Relative Velocity.x = DistChg * Unit.x - ( DirChg * pi/180 ) * Dist * Unit.y
   * Relative Velocity.y = DistChg * Unit.y + ( DirChg * pi/180 ) * Dist * Unit.x
   *
   * Unit = Relative Position / Dist
   *
   * Relative Position = Absolute Position - Player Position
   */
}

RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "FuzzyBallView.icc"
#endif
