/* -*- Mode: C++ -*-
 * $Id: FuzzyBallView.h,v 1.1.1.1 2001/04/08 11:33:59 sigge Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: FuzzyBallView.cc
 * Unitname: Framework/view
 * $Revision: 1.1.1.1 $
 * Created by: Mikael Brnnstrm, Andreas Sigfridsson, Andreas Bckert
 * Last modified by $Author: sigge $ $Date: 2001/04/08 11:33:59 $
 *
 *
 * HISTORY
 *
 * $Log: FuzzyBallView.h,v $
 * Revision 1.1.1.1  2001/04/08 11:33:59  sigge
 * Imported sources
 *
 * Revision 1.2  2000/11/19 16:46:49  mikbr212
 * now uses absolute vectors
 *
 * Revision 1.1  2000/11/19 11:38:50  mikbr212
 * created
 *
 */

/**
 * @file FuzzyBallView.h
 * Implementation file for the class FuzzyBallView.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.1.1.1 $
 * $Author: sigge $
 * $Date: 2001/04/08 11:33:59 $
 */



#ifndef FUZZYBALLVIEW_H
#define FUZZYBALLVIEW_H

#include "ViewWithHistory.h"
#include "ParameterView.h"
#include "AgentView.h"
#include "GameView.h"
#include "CommandView.h"
#include "view_constants.h"


RS_BEGIN_NAMESPACE

/**
 * @class FuzzyBallView FuzzyBallView.h
 * This view is used to keep track of the ball.
 *
 * <!-- @invariant The class invariant is... -->
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.1.1.1 $
 * $Author: sigge $
 * $Date: 2001/04/08 11:33:59 $
 */
class FuzzyBallView : public ViewWithHistory {
public:
  /** Create a ball view with ViewId @a id and the
    history length @a hist_len. */
  FuzzyBallView(const ViewId id,
		const UInt hist_len = DEFAULT_HISTORY_LENGTH);

  /** Destroys the ball view. */
  virtual ~FuzzyBallView();

  VectorUCO getAbsSpeed(const Int when = NOW) const;
  VectorUCO getAbsPosition(const Int when = NOW) const;

  // Relative to body
  VectorUCO getRelSpeed(const Int when = NOW) const;
  VectorUCO getRelPosition(const Int when = NOW) const;

  FloatUnknown getDistChange(const Int when = NOW) const;
  AngleDegUnknown getDirChange(const Int when = NOW) const;

  
protected:
  /** A pointer to the parameter view. */
  ParameterView* parameterView;

  /** A pointer to the game view. */
  GameView* gameView;

  /** A pointer to the command view. */
  CommandView* commandView;

  /** A pointer to the agent view. */
  AgentView* agentView;


  /** The history of the ball. */
  std::vector<VectorUCO> absSpeedHist;
  std::vector<VectorUCO> absPosHist;

  std::vector<VectorUCO> relSpeedHist;
  std::vector<VectorUCO> relPosHist;
  
  /** Observed history */
  std::vector<VectorUCO> obsRelSpeedHist;
  std::vector<VectorUCO> obsRelPosHist;

  // DEBUG
  std::vector<FloatUnknown> obsDistChangeHist;
  std::vector<AngleDegUnknown> obsDirChangeHist;
  
  virtual void myNewCycle();
  virtual void myUpdateAfterCoachMessage();
  virtual void myUpdateAfterInit();
  virtual void myUpdateAfterSee();
  virtual void myUpdateAfterHear();
  virtual void myUpdateAfterSense();
  virtual void myUpdateAfterCommand();

  /** Method for estimating the speed of the ball. */
  void estimate();
  void degradeVector(VectorUCO& vec, Float f);
  void fixUnknownToNoConfidence(VectorUCO& vec);

private:
  // Hide copy-constructor and assignment.
  FuzzyBallView(const FuzzyBallView&);
  FuzzyBallView& operator= (const FuzzyBallView&);  
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "FuzzyBallView.icc"
#endif


#endif // !FUZZYBALLVIEW_H

