/* -*- Mode: C++ -*-
 * $Id: FuzzyPlayerView.cc,v 1.2 2001/04/20 16:21:58 sigge Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: FuzzyPlayerView.cc
 * Unitname: Framework/view
 * $Revision: 1.2 $
 * Created by: Mikael Brnnstrm, Andreas Sigfridsson, Andreas Bckert
 * Last modified by $Author: sigge $ $Date: 2001/04/20 16:21:58 $
 *
 *
 * HISTORY
 *
 * $Log: FuzzyPlayerView.cc,v $
 * Revision 1.2  2001/04/20 16:21:58  sigge
 * Replaced getPlayerMode with getPlayerModes.
 *
 * Revision 1.1.1.1  2001/04/08 11:33:59  sigge
 * Imported sources
 *
 * Revision 1.1  2000/11/20 12:41:27  mikbr212
 * File was corrupt in the cvs repository. Removed and added file again.
 *
 * Revision 1.8  2000/11/19 17:14:52  mikbr212
 * added kick-stuff
 *
 * Revision 1.7  2000/11/19 16:50:32  mikbr212
 * fixed X and Y stuff in afterSee
 *
 * Revision 1.6  2000/11/19 16:46:49  mikbr212
 * now uses absolute vectors
 *
 * Revision 1.5  2000/11/19 15:32:36  mikbr212
 * fixed more stuff
 *
 * Revision 1.4  2000/11/19 15:23:50  mikbr212
 * added more stuff
 *
 * Revision 1.3  2000/11/19 12:51:00  mikbr212
 * added stuff
 *
 * Revision 1.2  2000/11/19 11:58:14  mikbr212
 * fixed stuff
 *
 * Revision 1.1  2000/11/19 11:38:49  mikbr212
 * created
 *
 */

/**
 * @file FuzzyPlayerView.cc
 * Implementation file for the class FuzzyPlayerView.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.2 $
 * $Author: sigge $
 * $Date: 2001/04/20 16:21:58 $
 */


#include "FuzzyPlayerView.h"

RS_BEGIN_NAMESPACE

FuzzyPlayerView::FuzzyPlayerView(const ViewId id,
		   const UInt hist_len)
  : ViewWithHistory(id, hist_len),
    parameterView(NULL), 
    agentView(NULL),
    playersHist(hist_len, std::vector<FuzzyPlayer> (NUMBER_OF_PLAYERS, FuzzyPlayer()))
{
  getView(parameterView, PARAMETER_VIEW_ID);
  getView(agentView, AGENT_VIEW_ID, parameterView, hist_len);
}


FuzzyPlayerView::~FuzzyPlayerView()
{
  viewManager->releaseView(AGENT_VIEW_ID);
  viewManager->releaseView(PARAMETER_VIEW_ID);
}


void FuzzyPlayerView::myUpdateAfterInit()
{

}


// NewCycle is called *before* the values in CSensorInterface
// are erased, so this is the last chance to use them...
void FuzzyPlayerView::myNewCycle() {
  agentView->newCycle(currentUpdateNumber);

  // Copy our last state to the current state.
  playersHist[getIndex(NEXT)] = playersHist[getIndex(NOW)];

  // Predict the movements of the players.
  for(int i = 0; i < NUMBER_OF_PLAYERS; i++) {
    playersHist[getIndex(NEXT)][i].predict();
  }

  increaseIndex();
}


void FuzzyPlayerView::myUpdateAfterSee()
{
  // Eval the callbackfunction. (Why a callback?!?!!)

  playersObs.clear();
  sensorInterface->forEachPlayer(&View::processPlayer, this);

  // Do a greedy search for the best match.

  int sz;
  if(sz = playersObs.size() > NUMBER_OF_PLAYERS)
  {
//     std::vector<FuzzyPlayer>::iterator it = playersObs.begin();
//     for(int i=0; i<sz - NUMBER_OF_PLAYERS; i++)
//       playersObs.pop_back();
//     playersObs.clear();
//       playersObs.erase(it++);
//     sz = playersObs.size();
    if(sz > NUMBER_OF_PLAYERS)
     cout << "playersObs.size() > NUMBER_OF_PLAYERS (it is" << sz << ")" << endl;
  }
  if(playersHist[getIndex(NOW)].size() > NUMBER_OF_PLAYERS)
    cout << "playersHist[getIndex(NOW)].size() > NUMBER_OF_PLAYERS (it is " << playersHist[getIndex(NOW)].size() << ")" << endl;
  
  // Calculate how well the observation matches the
  // knowledge base.
  FuzzyBool matchValues[NUMBER_OF_PLAYERS][NUMBER_OF_PLAYERS];
  for(unsigned int i = 0; i < playersObs.size(); i++) {
    FuzzyPlayer curObs = playersObs[i];

    for(unsigned int j = 0; j < playersHist[getIndex(NOW)].size(); j++) {
      matchValues[i][j] = playersHist[getIndex(NOW)][j].isMatch(curObs);
    }
  }

  for(unsigned int i = 0; i < playersObs.size(); i++) {
    FuzzyBool bestMatch = FuzzyBool(0.0);
    int bestMatchNum = 0;
    for(unsigned int j = 0; j < playersHist[getIndex(NOW)].size(); j++) {
      if(matchValues[i][j] > bestMatch) {
	bestMatch = matchValues[i][j];
	bestMatchNum = j;
      }
    }

    // Clear the players matchvalue to avoid duplicate matching.
    for(unsigned int j = 0; j < playersObs.size(); j++) {
      matchValues[j][bestMatchNum] = FuzzyBool(0.0);
    }

    playersHist[getIndex(NOW)][bestMatchNum] = FuzzyPlayer(playersHist[getIndex(NOW)][bestMatchNum],
							   playersObs[i]);
  }

  // Check the consistence of our KB.
  // Check for duplicate shirtnumbers.
  for(unsigned int i = 0; i < playersHist[getIndex(NOW)].size(); i++)
    {
      UInt16UCO refUNum = playersHist[getIndex(NOW)][i].getUNum();
      TeamUCO refTeam = playersHist[getIndex(NOW)][i].getTeam();
      for(unsigned int j = 0; j < playersHist[getIndex(NOW)].size(); j++)
	{
	  UInt16UCO curUNum = playersHist[getIndex(NOW)][j].getUNum();
	  TeamUCO curTeam = playersHist[getIndex(NOW)][j].getTeam();
	  if(refTeam == curTeam && i != j)
	    {
	      if (curUNum == refUNum)
		{
		  // Inconsitency found.
		  Confidence refConf = refUNum.getConfidence();
		  Confidence curConf = curUNum.getConfidence();
		  
		  if( refConf > curConf ) {
		    curUNum.setConfidence(Confidence(0.0));
		    playersHist[getIndex(NOW)][j].setUNum(curUNum);
		  } else {
		    refUNum.setConfidence(Confidence(0.0));
		    playersHist[getIndex(NOW)][i].setUNum(refUNum);
		    //		    break; // No need to parse further. might change!
		  }
		  
		}
	    }
	}
    }

  // Check for even team distribution.
  /*
  int nOpp = 0, nTeam = 0, nUnkn = 0;
  bool even = false;
  while(!even) {
    for(unsigned int i = 0; i < playersHist[getIndex(NOW)].size(); i++)
      {
	TeamUCO curTeam = playersHist[getIndex(NOW)][i].getTeam();
	if(curTeam.isKnown()) {
	  switch(curTeam.getValue().getValue().getValue())
	    {
	    case TB_OUR_TEAM:
	      nTeam++;
	      break;
	    case TB_THEIR_TEAM:
	      nOpp++;
	      break;
	    default:
	      // Shouldnt happen.
	      break;
	    }
	}
      }
    if(nOpp <= 11 
       && nTeam <= 11 
       && (nTeam + nOpp + nUnkn) <= 22)
      even = true;
    else
      even = false;
      } */
}

void FuzzyPlayerView::myUpdateAfterHear()
{
//    agentView->updateAfterHear(currentUpdateNumber);
//    //markerView->updateAfterHear(currentUpdateNumber);

//    PlayMode pm = gameView->getPlayModes(NOW);
//    if ( pm == PM_BEFORE_KICK_OFF ||
//         pm == PM_KICK_OFF ||
//         pm == PM_OUR_KICK_OFF ||
//         pm == PM_THEIR_KICK_OFF ) {
//      theBall[getIndex(NOW)].setAbsoluteSpeedVector(VectorUCO(0,0));
//      //theBall[getIndex(NOW)].setPosition(Point(0,0));
//      theBall[getIndex(NOW)].setVector(VectorUCO(agentView->getAgentPosition(NOW), PointUCO(0,0)).
//  				     rotate(-agentView->getFaceDirection(NOW)));

//    }
}


void FuzzyPlayerView::myUpdateAfterCommand()
{
  /* Since we currently send all commands last in the cycle, even
     immediate actions, we can do all the updating at the begining
     of the new cycle. */
}

void FuzzyPlayerView::processPlayer(const ServerPlayerObject& plr) 
{
  if(playersObs.size() >= NUMBER_OF_PLAYERS)
    return;			// RoboSoc is tricking us
  VectorUCO plrAbsSpeed;
  PointUCO plrAbsPos;
  AngleDegUCO plrDir;
  FloatUCO plrDist;

  FloatUnknown distChange = plr.distanceChange;
  AngleDegUnknown dirChange = plr.directionChange;

  // Calculate the position of plr.
  plrDir.setConfidence(Confidence(1.0));
  if(agentView->getFaceDirection(NOW).isKnown()) {
    plrDir = agentView->getFaceDirection(NOW).getAngle().getAngle().getAngle();
  } else {
    plrDir = 0.0;
    plrDir.setConfidence(Confidence(0.0));
  }

  if(plr.direction.isKnown()) {
    plrDir += plr.direction.getAngle();
  } else {
    plrDir.setConfidence(Confidence(0.0));
  }

  plrDist.setConfidence(Confidence(1.0));
  if(plr.distance.isKnown()) {
    plrDist = plr.distance;
  } else {
    plrDist.setConfidence(Confidence(0.0));
  }

  // We have the realtive position.
  plrAbsPos = VectorUCO(plrDir, plrDist);

  if(agentView->getAgentPosition(NOW).isKnown()) {
    plrAbsPos += agentView->getAgentPosition(NOW);
  } else {
    plrAbsPos = clearPointConf(plrAbsPos);
  }

  // plrAbsPos should be a playerposition with a valid confidence value.

  /*
   * Absolute Velocity = Agent Velocity + Relative Velocity
   *
   * Relative Velocity.x = DistChg * Unit.x - ( DirChg * pi/180 ) * Dist * Unit.y
   * Relative Velocity.y = DistChg * Unit.y + ( DirChg * pi/180 ) * Dist * Unit.x
   *
   * Unit = Relative Position / Dist
   *
   * Relative Position = Absolute Position - Player Position
   */


  if(distChange.isKnown() && dirChange.isKnown() && plrDir.isKnown()) {
    Point unit = Point(Vector(plrDir.getAngle().getAngle().getAngle(), 1)); 
    Float tmp = dirChange.getAngle().getRad() * plrDist.getValue().getValue().getValue();
    Point speed(distChange.getValue() * unit.getX() - tmp * unit.getY(),
		distChange.getValue() * unit.getY() + tmp * unit.getX());

    // Convert to absolute speed

    plrAbsSpeed = VectorUCO(speed);
    plrAbsSpeed.rotate(agentView->getNeckDirection(NOW)); 
    plrAbsSpeed.rotate(agentView->getBodyDirection(NOW));
    plrAbsSpeed += agentView->getSpeedVector(NOW);
  } else {
    plrAbsSpeed = VectorUCO(0,0);
    plrAbsSpeed = clearVectorConf(plrAbsSpeed);
  }

  // Create the reference player.

  UInt16UCO UNum;
  if(plr.uniformNumber.isKnown()) {
    UNum = plr.uniformNumber.getValue();
    UNum.setConfidence(Confidence(1.0));
  } else {
    UNum = 0;
    UNum.setConfidence(Confidence(0.0));
  }

  FuzzyPlayer curPlayer;
  curPlayer.setPos(plrAbsPos);
  curPlayer.setSpeed(plrAbsSpeed);
  curPlayer.setUNum(UNum);

  TeamUCO plrTeam = TeamUCO(TB_OUR_TEAM);
  plrTeam.setConfidence(Confidence(1.0));
  switch(plr.team) {
  case T_OUR_TEAM:
    plrTeam = TB_OUR_TEAM;
    break;
  case T_THEIR_TEAM:
    plrTeam = TB_THEIR_TEAM;
    break;
  default:
    plrTeam.setConfidence(Confidence(0.0));
    break;
  } 
  curPlayer.setTeam(plrTeam);

  curPlayer.fixUnknown();

  // Insert it into our list of player observations.
  playersObs.insert(playersObs.begin(), curPlayer);
}

PointUCO FuzzyPlayerView::clearPointConf(PointUCO &p)
{

  FloatUCO x = p.getX();
  x.setConfidence(Confidence(0.0));
  FloatUCO y = p.getY();
  y.setConfidence(Confidence(0.0));


  PointUCO res(x,y);

  return res;

  /*  p.setX(p.getX().setConfidence(Confidence(0.0)));
      p.setY(p.getY().setConfidence(Confidence(0.0))); */
  //  return p

}
VectorUCO FuzzyPlayerView::clearVectorConf(VectorUCO &v)
{
  AngleDegUCO dir = v.getAngle();
  FloatUCO dist = v.getLength();
  dir.setConfidence(0.0);
  dist.setConfidence(0.0);

  VectorUCO v2(dir,dist);
  return v2;
}



RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "FuzzyPlayerView.icc"
#endif
