/* -*- Mode: C++ -*-
 * $Id: PassTreeView.cc,v 1.19 2001/07/31 17:07:20 mikbr212 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: PassTreeView.cc
 * Unitname: Framework/view
 * $Revision: 1.19 $
 * Last modified by $Author: mikbr212 $ $Date: 2001/07/31 17:07:20 $
 *
 *
 * HISTORY
 *
 * $Log: PassTreeView.cc,v $
 * Revision 1.19  2001/07/31 17:07:20  mikbr212
 * Dynamic pass tree
 *
 * Revision 1.18  2001/07/31 11:20:35  sigge
 * New static passtree.
 *
 * Revision 1.17  2001/07/30 17:00:01  mikbr212
 * Debugged and fixed stupid static bug
 *
 * Revision 1.16  2001/07/27 13:19:06  sigge
 * Promote last pass tree. Probably not enough with 0.1.
 *
 * Revision 1.15  2001/07/26 12:20:47  mikbr212
 * Changed passtree completly
 *
 * Revision 1.14  2001/07/06 10:00:38  mikbr212
 * Merged updates and resolved conflicts (a lot)
 *
 * Revision 1.13  2001/06/19 14:01:00  mikbr212
 * Fixed circular includes stuff. Somebody should check that I did it right.
 *
 * Revision 1.12  2001/04/24 19:13:43  mikbr212
 * No idea what i have done
 *
 * Revision 1.11  2001/04/23 22:50:09  mikbr212
 * added their passtree without agent
 *
 * Revision 1.10  2001/04/23 14:44:03  mikbr212
 * ??
 *
 * Revision 1.9  2001/04/21 16:26:52  mikbr212
 * fixed stuff
 *
 * Revision 1.8  2001/04/20 12:03:41  mikbr212
 * added display
 *
 * Revision 1.7  2001/04/20 11:45:07  mikbr212
 * Uses PassTree now
 *
 * Revision 1.6  2001/04/19 13:24:18  mikbr212
 * fixed stuff
 *
 * Revision 1.5  2001/04/19 13:23:35  mikbr212
 * fixed stuff
 *
 * Revision 1.4  2001/04/19 12:28:09  mikbr212
 * new display
 *
 * Revision 1.3  2001/04/18 14:42:43  mikbr212
 * made better
 *
 * Revision 1.2  2001/04/17 13:52:31  mikbr212
 * made it work
 *
 * Revision 1.1.1.1  2001/04/08 11:33:59  sigge
 * Imported sources
 *
 * Revision 1.1  2000/11/03 22:09:12  mikbr212
 * Created (not working yet)
 *
 * Revision 1.8  2000/10/28 20:22:02  mikbr212
 */

/**
 * @file PassTreeView.cc
 * Implementation file for the view class PassTreeView.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.19 $
 * $Author: mikbr212 $
 * $Date: 2001/07/31 17:07:20 $
 */

#include "ParameterView.h"
#include "WorldStateView.h"
#include "Display.h"

#include <sys/time.h>
#include "PassTreeView.h"

RS_BEGIN_NAMESPACE

/**
 * Constructor. Use getView instead.
 * @see getView
 */
PassTreeView::PassTreeView(const ViewId id)
  : View(id),
    parameterView(NULL),
    worldStateView(NULL),
    communicationView(NULL)
{
  getView(parameterView, PARAMETER_VIEW_ID);
  getView(worldStateView, WORLD_STATE_VIEW_ID);
  getView(communicationView, COMMUNICATION_VIEW_ID);

  // Inner defenders
  promotedPassWaysHigh.push_back(PassTree::PassWay(2, 8));
  promotedPassWaysHigh.push_back(PassTree::PassWay(3, 9));
  promotedPassWaysLow.push_back(PassTree::PassWay(2, 6));
  promotedPassWaysLow.push_back(PassTree::PassWay(3, 7));

  // Outer defenders
  promotedPassWaysHigh.push_back(PassTree::PassWay(4, 8));
  promotedPassWaysHigh.push_back(PassTree::PassWay(5, 9));

  // Flank defenders
  promotedPassWaysHigh.push_back(PassTree::PassWay(6, 8));
  promotedPassWaysHigh.push_back(PassTree::PassWay(7, 9));

  // Offenders
  promotedPassWaysHigh.push_back(PassTree::PassWay(8, 10));
  promotedPassWaysHigh.push_back(PassTree::PassWay(9, 11));
  promotedPassWaysHigh.push_back(PassTree::PassWay(10, 11));
  promotedPassWaysHigh.push_back(PassTree::PassWay(11, 10));
  promotedPassWaysMed.push_back(PassTree::PassWay(8, 9));
  promotedPassWaysMed.push_back(PassTree::PassWay(9, 8));
  promotedPassWaysMed.push_back(PassTree::PassWay(8, 11));
  promotedPassWaysMed.push_back(PassTree::PassWay(9, 11));
  promotedPassWaysLow.push_back(PassTree::PassWay(10, 8));
  promotedPassWaysLow.push_back(PassTree::PassWay(11, 9));
  promotedPassWaysLow.push_back(PassTree::PassWay(10, 9));
  promotedPassWaysLow.push_back(PassTree::PassWay(11, 8));
}

PassTreeView::~PassTreeView()
{
  viewManager->releaseView(PARAMETER_VIEW_ID);
  viewManager->releaseView(WORLD_STATE_VIEW_ID);
  viewManager->releaseView(COMMUNICATION_VIEW_ID);
}

void PassTreeView::myNewCycle() {
  worldStateView->newCycle(currentUpdateNumber);
  communicationView->newCycle(currentUpdateNumber);
  
  const WorldState& ws(worldStateView->getWorldState());
  const BallState& ball(ws.getBall());

  if(ball.getC() > 0.1) {
    agentPassTree = PassTree(ws);
//      agentPassTree.promotePassWays(promotedPassWaysHigh, 1.0);
//      agentPassTree.promotePassWays(promotedPassWaysMed, 0.8);
//      agentPassTree.promotePassWays(promotedPassWaysLow, 0.6);
//      agentPassTree.setStatic(true);
    agentPassTree.build();
    agentPassTree.getAnalysis(); // build cache
//      promotedPassWays = agentPassTree.getPassWays();
  } //  else {
//      promotedPassWays.clear();
//    }
  agentPassTree.display();
}

Message PassTreeView::passWaysToMessage(const 
					std::vector<PassTree::PassWay>& pw) {
  std::string str(pw.size(),'x');
  unsigned int from, to, i;
  unsigned char ch;
  for(i = 0; i < pw.size(); i++) {
    from = (unsigned int) pw[i].getFromUniformNumber();
    to = (unsigned int) pw[i].getToUniformNumber();
    ch = (from << 4) | to;
    str[i] = ch;
  }
  return Message(passWaysMessageType, str);
}

std::vector<PassTree::PassWay> PassTreeView::messageToPassWays(const 
							       Message& msg) {
  if(msg.getType() != passWaysMessageType)
    return std::vector<PassTree::PassWay>();

  std::string str(msg.getContents());
  std::vector<PassTree::PassWay> pw(str.length());
  unsigned int from, to, i;
  unsigned char ch;
  for(i = 0; i < str.length(); i++) {
    ch = str[i];
    from = (unsigned int) ((ch & 0xf0) >> 4);
    to = (unsigned int) (ch & 0x0f);
    pw[i] = PassTree::PassWay(from,to);
  }
  return pw;
}



RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "PassTreeView.icc"
#endif

