/* -*- Mode: C++ -*-
 * $Id: PassTreeView.h,v 1.9 2001/07/31 11:20:35 sigge Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: PassTreeView.h
 * Unitname: Framework/view
 * $Revision: 1.9 $
 * Last modified by $Author: sigge $ $Date: 2001/07/31 11:20:35 $
 *
 *
 * HISTORY
 *
 * $Log: PassTreeView.h,v $
 * Revision 1.9  2001/07/31 11:20:35  sigge
 * New static passtree.
 *
 * Revision 1.8  2001/07/26 12:20:47  mikbr212
 * Changed passtree completly
 *
 * Revision 1.7  2001/07/06 10:00:38  mikbr212
 * Merged updates and resolved conflicts (a lot)
 *
 * Revision 1.6  2001/06/19 14:01:00  mikbr212
 * Fixed circular includes stuff. Somebody should check that I did it right.
 *
 * Revision 1.5  2001/04/23 22:50:09  mikbr212
 * added their passtree without agent
 *
 * Revision 1.4  2001/04/20 11:45:07  mikbr212
 * Uses PassTree now
 *
 * Revision 1.3  2001/04/18 14:42:43  mikbr212
 * made better
 *
 * Revision 1.2  2001/04/17 13:52:31  mikbr212
 * made it work
 *
 * Revision 1.1.1.1  2001/04/08 11:33:59  sigge
 * Imported sources
 *
 * Revision 1.1  2000/11/03 22:09:13  mikbr212
 * Created (not working yet)
 *
 */

/**
 * @file PassTreeView.h
 * Include file for the view class PassTreeView.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.9 $
 * $Author: sigge $
 * $Date: 2001/07/31 11:20:35 $
 */


#ifndef PASSTREEVIEW_H
#define PASSTREEVIEW_H


#include "View.h"

//  #include "ParameterView.h"
//  #include "WorldStateView.h"

#include "view_constants.h"
#include "ViewManager.h"
#include "TLine.h"
#include "TVector.h"
//  #include "Display.h"

#include "PassTree.h"

#include <vector>
#include <algorithm>
#include "Message.h"
#include "CommunicationView.h"

RS_BEGIN_NAMESPACE

class ParameterView;
class WorldStateView;

/**
 * @class PassTreeView PassTreeView.h
 * @par Unit:
 *      Framework/view
 * $Revision: 1.9 $
 * $Author: sigge $
 * $Date: 2001/07/31 11:20:35 $
 */


class PassTreeView : public View {
public:
  PassTreeView(const ViewId id);

  /** Destroys the view. */
  ~PassTreeView();

  const PassTree& getAgentPassTree() const;

protected:
  Message passWaysToMessage(const std::vector<PassTree::PassWay>& pw);
  std::vector<PassTree::PassWay> messageToPassWays(const Message& msg);

  virtual void myNewCycle();
  virtual void myUpdateAfterCoachMessage();
  virtual void myUpdateAfterInit();
  virtual void myUpdateAfterSee();
  virtual void myUpdateAfterHear();
  virtual void myUpdateAfterSense();
  virtual void myUpdateAfterCommand();
  
  ParameterView* parameterView;
  WorldStateView* worldStateView;
  CommunicationView* communicationView;
  
  PassTree agentPassTree;
  std::vector<PassTree::PassWay> promotedPassWaysHigh;
  std::vector<PassTree::PassWay> promotedPassWaysMed;
  std::vector<PassTree::PassWay> promotedPassWaysLow;

private:
  // Hide copy-constructor and assignment.
  PassTreeView(const PassTreeView&);
  PassTreeView& operator= (const PassTreeView&);  
};


RS_END_NAMESPACE

/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "PassTreeView.icc"
#endif

#endif // !PASSTREEVIEW_H
