/* -*- Mode: C++ -*-
 * $Id: PlayerState.h,v 1.33 2001/07/28 15:42:57 mikbr212 Exp $
 * 
 * HISTORY 
 *
 * $Log: PlayerState.h,v $
 * Revision 1.33  2001/07/28 15:42:57  mikbr212
 * Added new doKickToSpeedAvoidCollisions
 *
 * Revision 1.32  2001/07/28 06:51:12  mikbr212
 * Removed compile errors
 *
 * Revision 1.31  2001/07/27 15:52:08  andbo928
 * Fixed a ton of warnings. NR intercept thingy added (but not used).
 *
 * Revision 1.30  2001/07/26 12:40:58  mikbr212
 * removed PlayerState::getTeam()
 *
 * Revision 1.29  2001/07/06 10:00:38  mikbr212
 * Merged updates and resolved conflicts (a lot)
 *
 * Revision 1.28  2001/07/05 17:11:53  andbo928
 * Removed get/setTeam, added Agent=Player assignemnt opertaor.
 *
 * Revision 1.27  2001/07/04 12:09:38  andbo928
 * Removed some unused attributes.
 *
 * Revision 1.26  2001/07/02 14:45:33  andbo928
 * Added canIntercept method.
 *
 * Revision 1.25  2001/07/01 14:15:05  sigge
 * Added Mickes code (Statistics view)
 *
 * Revision 1.24  2001/07/01 12:38:40  andbo928
 * Added new limitations imposed by the new server in simulation. Added doChangeViewFoo methods.
 *
 * Revision 1.23  2001/06/19 14:02:09  mikbr212
 * Added set and get methods for intention and team.
 *
 * Revision 1.22  2001/06/05 20:33:53  mikbr212
 * Fixed doxygen-stubs for all classes
 *
 */

/**
 * @file PlayerState.h
 * Definition file for the classes PlayerState and AgentState.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.33 $
 * $Author: mikbr212 $
 * $Date: 2001/07/28 15:42:57 $
 */

#ifndef PLAYERSTATE_H
#define PLAYERSTATE_H

#include "library_types.h"
#include "ViewManager.h"
#include "ParameterView.h"
#include "view_constants.h"
#include "PlayerObject.h"
#include "AgentObject.h"
#include "TVector.h"
#include "TAngle.h"
#include "State.h"
#include "Step.h"
//#include "Intention.h"

RS_BEGIN_NAMESPACE

class BallState; // To avoid circular references.
class Intention; // To avoid circular references.

/**
 * @class PlayerState PlayerState.h
 * Text...
 */

class PlayerState : public State
{
public:
  // Nested types.
  enum cmdType {cmdTurn, cmdDash, cmdKick, cmdMove, cmdTurnNeck, 
		cmdChangeView, cmdNull};
  
private:
  // Private methods/attributes.
  enum viewWidth {vwNarrow, vwNormal, vwWide};
  enum viewQuality {vqHigh, vqLow};
  
  int getInterceptTimeNR(const BallState& ball, int tStep) const;
  float nr_f(float t, float A, float B, float C, float D, float E) const;
  float nr_fdif(float t, float A, float B, float C, float D, float E) const;
  float tToS(float t) const;
  float sToT(float s) const;

protected:
  // Protected methods/attributes.

  TVector speed;
  TAngle direction;
  TAngle neckDirection;
  TVector position;
  int uniformNumber;
  int gameTime;

  Intention* intention;

  cmdType nextCommand; // Turn/Kick/Dash-type commands.
  cmdType nextCommand2; // TurnNeck commands.
  cmdType nextCommand3; // ViewMode commands.

  cmdType lastCommand;

  double turnAngle;
  double dashPower;
  TVector kickVector;
  double turnNeckAngle;
  viewWidth vWidth;
  viewQuality vQuality;

  // Some uesful views.
//    static ParameterView* parameterView;
  ParameterView* parameterView;

  TAngle getActualTurn(TAngle angle);


public:
  // Public methods/attributes.
  bool overlaps(const BallState& other) const;
  bool overlaps(const PlayerState& other) const;

  PlayerState();
  bool PlayerState::operator==(const PlayerState& other) const;
  void init(const PlayerObject& playerObject);

  void merge(const PlayerState& other);

  Step getStep();

  double getKickPowerRate(const BallState& ball) const;
  bool isBallKickable(const BallState& ball) const;
  int getInterceptTime(const BallState& ball) const;
  bool canIntercept(const BallState& ball) const;

  virtual bool doKick(const TVector& v, BallState& ball);
  virtual bool doKickToSpeed(const TVector& speed, BallState& ball);
  virtual bool doKickToPosition(const TVector& position, BallState& ball);
  virtual bool doKickToSpeedAvoidCollisions(const TVector& speed, 
					    BallState& ball);

  virtual bool doDash(float power);
  virtual bool doDashDistance(float dist);
  virtual bool doDashToSpeed(float speed);
  virtual bool doTurn(float angle);
  virtual bool doActualTurn(float angle);
  virtual bool doTurnTo(float dir);
  virtual bool doTurnTowards(const TVector& position);

  virtual bool doTurnNeck(float angle);
  virtual bool doTurnNeckTo(float dir);
  virtual bool doTurnNeckTowards(const TVector& position);
  virtual bool doNull();

  virtual bool doChangeViewWide();
  virtual bool doChangeViewNormal();
  virtual bool doChangeViewNarrow();
  virtual bool doChangeViewHigh();
  virtual bool doChangeViewLow();

  TVector getSpeed() const;
  void setSpeed(const TVector &value);
  double getDirection() const;
  void setDirection(double value);
  TVector getDirectionVector() const;

  double getNeckDirection() const;
  void setNeckDirection(double value);
  TVector getNeckDirectionVector() const;

  inline TVector getPosition() const { return position; }
  inline void setPosition(const TVector &value) {   position = value; }

  int getGameTime() const;
  void setGameTime(int _gameTime);
  
  int getUniformNumber() const;
  void setUniformNumber(int value);

  Intention* getIntention() const;
  void setIntention(Intention* _intention);

  double getC() const;
  void setC(double _c);

  cmdType getLastCommandType() const;

  // From State
  virtual void step();
  // Returns number of chars needed.
  virtual string serialize() const;
  // Returns number of chars consumed.
  virtual string deserialize(const string& buf);
};


/**
 * @class AgentState PlayerState.h
 * Text...
 */

class AgentState : public PlayerState
{
public:
  // Nested types.

private:
  // Private methods/attributes.

protected:
  // Protected methods/attributes.
  float stamina;
  float effort;
  float recovery;

public:
  AgentState();
  void init(const AgentObject& agentObject);
  bool operator==(const AgentState& other) const;
  const AgentState& operator=(const PlayerState& other);

  // Public methods/attributes.
  float getStamina() const;
  void setStamina(float value);
  float getEffort() const;
  void setEffort(float value);
  float getRecovery() const;
  void setRecovery(float value);

  // From State
  virtual void step();
  // Returns number of chars needed.
  virtual string serialize() const;
  // Returns number of chars consumed.
  virtual string deserialize(const string& buf);
};

RS_END_NAMESPACE
#endif // PLAYERSTATE_H
