/* -*- Mode: C++ -*-
 * $Id: TVector.cc,v 1.16 2001/06/05 20:33:53 mikbr212 Exp $
 *
 * IDENTIFICATION
 *
 * Filename: WorldStateView.cc
 * Unitname: Framework/view
 * $Revision: 1.16 $
 * Created by: Mikael Brnnstrm, Andreas Sigfridsson, Andreas Bckert
 * Last modified by $Author: mikbr212 $ $Date: 2001/06/05 20:33:53 $
 *
 *
 * HISTORY
 *
 * $Log: TVector.cc,v $
 * Revision 1.16  2001/06/05 20:33:53  mikbr212
 * Fixed doxygen-stubs for all classes
 *
 * Revision 1.15  2001/04/21 11:23:40  mikbr212
 * Made more methods inline
 *
 * Revision 1.14  2001/04/14 08:40:00  mikbr212
 * removed warnings
 *
 * Revision 1.13  2001/04/13 17:33:33  mikbr212
 * added +=, -=, *=, /=
 *
 * Revision 1.12  2001/04/13 08:16:47  andbo928
 * Removed DOS-style returns.
 *
 * Revision 1.11  2001/04/12 16:02:33  mikbr212
 * Added icc and static stuff
 *
 *
 */

/**
 * @file TVector.cc
 * Implementation file for the class TVector.
 *
 * $Revision: 1.16 $
 * $Author: mikbr212 $
 * $Date: 2001/06/05 20:33:53 $
 */

#include <math.h>
#include "TVector.h"
#include <iostream>


RS_BEGIN_NAMESPACE
TVector::TVector()
  : x(0),
    y(0),
    c(0)
{
  x = 0;
  y = 0;
  c = 0;
}

TVector::TVector(double _x, double _y)
  : x(_x),
    y(_y),
    c(1)
{
}

TVector::TVector(double _x, double _y, double _c)
  : x(_x),
    y(_y),
    c(_c)
{
}

TVector TVector::TPolarVector(double r, double a)
{
  TVector res;
  res.setPolar(r, a);
  res.setC(1.0);
  return res;
}

TVector::TVector(const Vector& other)
{
  setPolar(other.getLength(), other.getAngle().getDeg());
  c = 1.0;
}

TVector::TVector(const VectorUnknown& other)
{
  if( other.isKnown() )
    {
      setPolar(other.getVector().getLength(), 
	       other.getVector().getAngle().getDeg());
      c = 1.0;
    }
  else
    {
      x = y = c = 0;
    }
}

TVector::TVector(const VectorUC& other)
{
  if( other.isKnown() )
    {
      setPolar(other.getVector().getLength(), 
	       other.getVector().getAngle().getDeg());
      c = other.getLength().getConfidence().getConfidence();
    }
  else
    {
      x = y = c = 0;
    }
}

TVector::TVector(const VectorUCO& other)
{
  if( other.isKnown() )
    {
      setPolar(other.getVector().getLength(), 
	       other.getVector().getAngle().getDeg());
      c = other.getLength().getConfidence().getConfidence();
    }
  else
    {
      x = y = c = 0;
    }
}

TVector::TVector(const Point& other)
{
  x = other.getX();
  y = other.getY();
}

TVector::TVector(const PointUnknown& other)
{
  if(other.isKnown())
    {
      x = other.getPoint().getX();
      y = other.getPoint().getY();
      c = 1.0;
    }
  else
    {
      x = y = c = 0;
    }
}

TVector::TVector(const PointUC& other)
{
  if(other.isKnown())
    {
      x = other.getPoint().getX();
      y = other.getPoint().getY();
      c = other.getX().getConfidence().getConfidence();
    }
  else
    { 
      x = y = c = 0;
    }
}

TVector::TVector(const PointUCO& other)
{
  if(other.isKnown())
    {
      x = other.getPoint().getX();
      y = other.getPoint().getY();
      c = other.getX().getConfidence().getConfidence();
    }
  else
    {
      x = y = c = 0;
    }
}

const TVector& TVector::setPolar(double r, double a)
{
  double rad = degToRad(a);
  setX(r*std::cos(rad));
  setY(r*std::sin(rad));
  return *this;
}

const TVector& TVector::rotate(double a)
{
  setPolar(getLength(), getAngle() + a);
  return *this;
}

bool TVector::confCompare(const TVector& other) const
{
  if(c > other.c)
    return true;
  return false;
}

TVector TVector::confSelect(const TVector& other) const
{
  if(confCompare(other))
    return *this;
  return other;
}

bool TVector::operator==(const TVector& other) const
{
  return 
    x == other.x
    && y == other.y
    && c == other.c;
}

std::ostream& operator<<(std::ostream& os, const TVector& vec)
{
  os << "(x: " << vec.getX() 
     << ", y: " << vec.getY() 
     << ", c: " << vec.getC() << ")";
  return os;
}

RS_END_NAMESPACE

/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "TVector.icc"
#endif

